"""A collection of functions which are triggered automatically by finder when
numpy package is included.
"""


from __future__ import annotations

import json
import os
import sys
from importlib.machinery import EXTENSION_SUFFIXES
from pathlib import Path
from textwrap import dedent

from .._compat import IS_CONDA, IS_MINGW, IS_WINDOWS
from ..finder import ModuleFinder
from ..module import Module

# The sample/pandas is used to test.
# Using pip (pip install numpy) in Windows/Linux/macOS from pypi (w/ OpenBLAS)
# And using cgohlke/numpy-mkl.whl, numpy 1.23.5+mkl in windows:
# https://github.com/cgohlke/numpy-mkl.whl/releases/download/v2023.1.4/numpy-1.23.5+mkl-cp311-cp311-win_amd64.whl
#
# Read the numpy documentation, especially if using conda-forge and MKL:
# https://numpy.org/install/#numpy-packages--accelerated-linear-algebra-libraries
#
# For conda-forge we can use the default installation with MKL:
# conda install -c conda-forge numpy
# Or use OpenBLAS:
# conda install -c conda-forge blas=*=openblas numpy


def load_numpy(finder: ModuleFinder, module: Module) -> None:
    """The numpy must be loaded as a package.

    Supported pypi and conda-forge versions (tested from 1.21.2 to 1.24.3).
    """
    finder.include_package("numpy")
    module.in_file_system = 1  # TODO: support zip or optimized mode
    # exclude the tests
    finder.exclude_module("numpy.compat.tests")
    finder.exclude_module("numpy.core.tests")
    finder.exclude_module("numpy.distutils.tests")
    finder.exclude_module("numpy.f2py.tests")
    finder.exclude_module("numpy.fft.tests")
    finder.exclude_module("numpy.lib.tests")
    finder.exclude_module("numpy.linalg.tests")
    finder.exclude_module("numpy.ma.tests")
    finder.exclude_module("numpy.matrixlib.tests")
    finder.exclude_module("numpy.polynomial.tests")
    finder.exclude_module("numpy.random._examples")
    finder.exclude_module("numpy.random.tests")
    finder.exclude_module("numpy.tests")
    finder.exclude_module("numpy.typing.tests")


def load_numpy__distributor_init(finder: ModuleFinder, module: Module) -> None:
    """Fix the location of dependent files in Windows."""
    if not (IS_WINDOWS or IS_MINGW):
        # In Linux and macOS it is detected correctly.
        return

    # patch the code when necessary
    code_string = module.file.read_text(encoding="utf-8")

    # installed from pypi, using zip_include_packages we must fix it
    numpy_dir = module.file.parent
    libs_dir = numpy_dir / ".libs"
    if libs_dir.is_dir():
        if module.in_file_system == 0:  # zip_include_packages
            # copy any file at site-packages/numpy/.libs
            finder.include_files(libs_dir, "lib/numpy/.libs")
    else:
        # cgohlke/numpy-mkl.whl, numpy 1.23.5+mkl
        libs_dir = numpy_dir / "DLLs"
        if libs_dir.is_dir():
            finder.exclude_module("numpy.DLLs")
            finder.include_files(libs_dir, "lib/numpy/DLLs")
        # conda-forge
        elif IS_CONDA:
            prefix = Path(sys.prefix)
            conda_meta = prefix / "conda-meta"
            packages = ["libblas", "libcblas", "liblapack"]
            blas_options = ["libopenblas", "mkl"]
            packages += blas_options
            files_to_copy: list[Path] = []
            for package in packages:
                try:
                    pkg = next(conda_meta.glob(f"{package}-*.json"))
                except StopIteration:
                    continue
                files = json.loads(pkg.read_text(encoding="utf-8"))["files"]
                files_to_copy += [
                    prefix / file
                    for file in files
                    if file.lower().endswith(".dll")
                ]
                blas = package
            for source in files_to_copy:
                finder.include_files(source, f"lib/{blas}/{source.name}")
            numpy_blas = f"""
            def init_numpy_blas():
                import os

                blas_path = os.path.join(
                    os.path.dirname(os.path.dirname(__file__)), "{blas}"
                )
                try:
                    os.add_dll_directory(blas_path)
                except (OSError, AttributeError):
                    pass
                env_path = os.environ.get("PATH", "").split(os.pathsep)
                if blas_path not in env_path:
                    env_path.insert(0, blas_path)
                    os.environ["PATH"] = os.pathsep.join(env_path)

            init_numpy_blas()
            """
            code_string += dedent(numpy_blas)

    # do not check dependencies already handled
    extension = EXTENSION_SUFFIXES[0]
    for file in numpy_dir.rglob(f"*{extension}"):
        finder.exclude_dependent_files(file)

    if module.in_file_system == 0:
        code_string = code_string.replace(
            "__file__", "__file__.replace('library.zip/', '')"
        )
    module.code = compile(code_string, os.fspath(module.file), "exec")


def load_numpy_core_numerictypes(
    finder: ModuleFinder, module: Module  # noqa: ARG001
) -> None:
    """The numpy.core.numerictypes module adds a number of items to itself
    dynamically; define these to avoid spurious errors about missing
    modules.
    """
    module.global_names.update(
        [
            "bool_",
            "cdouble",
            "complexfloating",
            "csingle",
            "double",
            "float64",
            "float_",
            "inexact",
            "intc",
            "int32",
            "number",
            "single",
        ]
    )


def load_numpy_distutils_command_scons(
    finder: ModuleFinder, module: Module  # noqa: ARG001
) -> None:
    """The numpy.distutils.command.scons module optionally imports the numscons
    module; ignore the error if the module cannot be found.
    """
    module.ignore_names.add("numscons")


def load_numpy_distutils_misc_util(
    finder: ModuleFinder, module: Module  # noqa: ARG001
) -> None:
    """The numpy.distutils.misc_util module optionally imports the numscons
    module; ignore the error if the module cannot be found.
    """
    module.ignore_names.add("numscons")


def load_numpy_distutils_system_info(
    finder: ModuleFinder, module: Module  # noqa: ARG001
) -> None:
    """The numpy.distutils.system_info module optionally imports the Numeric
    module; ignore the error if the module cannot be found.
    """
    module.ignore_names.add("Numeric")


def load_numpy_f2py___version__(
    finder: ModuleFinder, module: Module  # noqa: ARG001
) -> None:
    """The numpy.f2py.__version__ module optionally imports the __svn_version__
    module; ignore the error if the module cannot be found.
    """
    module.ignore_names.add("__svn_version__")


def load_numpy_linalg(
    finder: ModuleFinder, module: Module  # noqa: ARG001
) -> None:
    """The numpy.linalg module implicitly loads the lapack_lite module; make
    sure this happens.
    """
    finder.include_module("numpy.linalg.lapack_lite")


def load_numpy_random_mtrand(
    finder: ModuleFinder, module: Module  # noqa: ARG001
) -> None:
    """The numpy.random.mtrand module is an extension module and the numpy
    module imports * from this module; define the list of global names
    available to this module in order to avoid spurious errors about missing
    modules.
    """
    module.global_names.update(["rand", "randn"])



from __future__ import annotations

import clr

clr.AddReference("System.Windows.Forms")
import System.Windows.Forms as WinForms  # noqa
from System.Drawing import Point, Size  # noqa


class HelloApp(WinForms.Form):
    """A simple hello world app that demonstrates the essentials of
    winforms programming and event-based programming in Python."""

    def __init__(self):
        super().__init__()
        self.Text = "Hello World From Python"
        self.AutoScaleBaseSize = Size(5, 13)
        self.ClientSize = Size(392, 117)
        h = WinForms.SystemInformation.CaptionHeight
        self.MinimumSize = Size(392, (117 + h))

        # Create the button
        self.button = WinForms.Button()
        self.button.Location = Point(160, 64)
        self.button.Size = Size(820, 20)
        self.button.TabIndex = 2
        self.button.Text = "Click Me!"

        # Register the event handler
        self.button.Click += self.button_Click

        # Create the text box
        self.textbox = WinForms.TextBox()
        self.textbox.Text = "Hello World"
        self.textbox.TabIndex = 1
        self.textbox.Size = Size(1260, 40)
        self.textbox.Location = Point(160, 24)

        # Add the controls to the form
        self.AcceptButton = self.button
        self.Controls.Add(self.button)
        self.Controls.Add(self.textbox)

    def button_Click(self, sender, args):
        """Button click event handler"""
        print("Click")
        WinForms.MessageBox.Show("Please do not press this button again.")

    def run(self):
        WinForms.Application.Run(self)


def main():
    form = HelloApp()
    print("form created")
    app = WinForms.Application
    print("app referenced")
    app.Run(form)


if __name__ == "__main__":
    main()



"""A setup script to demonstrate build using orjson"""
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

from __future__ import annotations

from cx_Freeze import Executable, setup

setup(
    name="test_orjson",
    version="0.1",
    description="cx_Freeze script to test orjson",
    executables=[Executable("test_orjson.py")],
    options={"build_exe": {"excludes": ["tkinter"]}},
)



"""PyQt6 WebEngineWidgets Example"""

from __future__ import annotations

import sys

from PyQt6.QtCore import QUrl
from PyQt6.QtGui import QIcon
from PyQt6.QtWebEngineCore import QWebEnginePage
from PyQt6.QtWebEngineWidgets import QWebEngineView
from PyQt6.QtWidgets import (
    QApplication,
    QLineEdit,
    QMainWindow,
    QPushButton,
    QToolBar,
)


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.setWindowTitle("PyQt6 WebEngineWidgets Example")

        self.toolBar = QToolBar()
        self.addToolBar(self.toolBar)
        self.backButton = QPushButton()
        self.backButton.setIcon(
            QIcon(":/qt-project.org/styles/commonstyle/images/left-32.png")
        )
        self.backButton.clicked.connect(self.back)
        self.toolBar.addWidget(self.backButton)
        self.forwardButton = QPushButton()
        self.forwardButton.setIcon(
            QIcon(":/qt-project.org/styles/commonstyle/images/right-32.png")
        )
        self.forwardButton.clicked.connect(self.forward)
        self.toolBar.addWidget(self.forwardButton)

        self.addressLineEdit = QLineEdit()
        self.addressLineEdit.returnPressed.connect(self.load)
        self.toolBar.addWidget(self.addressLineEdit)

        self.webEngineView = QWebEngineView()
        self.setCentralWidget(self.webEngineView)
        initialUrl = "https://marcelotduarte.github.io/cx_Freeze/"
        self.addressLineEdit.setText(initialUrl)
        self.webEngineView.load(QUrl(initialUrl))
        self.webEngineView.page().titleChanged.connect(self.setWindowTitle)
        self.webEngineView.page().urlChanged.connect(self.urlChanged)

    def load(self):
        url = QUrl.fromUserInput(self.addressLineEdit.text())
        if url.isValid():
            self.webEngineView.load(url)

    def back(self):
        self.webEngineView.page().triggerAction(QWebEnginePage.Back)

    def forward(self):
        self.webEngineView.page().triggerAction(QWebEnginePage.Forward)

    def urlChanged(self, url):
        self.addressLineEdit.setText(url.toString())


if __name__ == "__main__":
    app = QApplication(sys.argv)
    mainWin = MainWindow()
    availableGeometry = mainWin.screen().availableGeometry()
    mainWin.resize(
        availableGeometry.width() * 2 // 3, availableGeometry.height() * 2 // 3
    )
    mainWin.show()
    sys.exit(app.exec())



"""Code to test that the plist_items option is working correctly."""

from __future__ import annotations

import plistlib
import sys
from pathlib import Path

import pytest
from data import BUILD_DIR, BUNDLE_NAME, TEST_KEY, TEST_VALUE

from cx_Freeze.sandbox import run_setup

FIXTURE_DIR = Path(__file__).resolve().parent


@pytest.mark.skipif(sys.platform != "darwin", reason="Macos tests")
@pytest.mark.datafiles(FIXTURE_DIR)
def test_plist_items(datafiles: Path):
    """Test that the plist_items option is working correctly."""
    run_setup(datafiles / "setup.py", ["bdist_mac"])
    # Test that the additional keys were correctly added to the plist.
    path = datafiles / f"{BUILD_DIR}/{BUNDLE_NAME}.app/Contents/Info.plist"
    contents = plistlib.loads(path.read_bytes())
    assert contents[TEST_KEY] == TEST_VALUE



"""Test ModuleFinder."""

import os
import py_compile
import sys
from importlib.machinery import BYTECODE_SUFFIXES, SOURCE_SUFFIXES

import pytest
from generate_samples import (
    absolute_import_test,
    bytecode_test,
    coding_default_utf8_test,
    coding_explicit_cp1252_test,
    coding_explicit_utf8_test,
    create_package,
    extended_opargs_test,
    maybe_test,
    maybe_test_new,
    package_test,
    relative_import_test,
    relative_import_test_2,
    relative_import_test_3,
    relative_import_test_4,
    same_name_as_bad_test,
    sub_package_test,
)

from cx_Freeze import ConstantsModule, ModuleFinder

# Each test description is a list of 5 items:
#
# 1. a module name that will be imported by ModuleFinder
# 2. a list of module names that ModuleFinder is required to find
# 3. a list of module names that ModuleFinder should complain
#    about because they are not found
# 4. a list of module names that ModuleFinder should complain
#    about because they MAY be not found
# 5. a string specifying packages to create; the format is obvious imo.
#
# Each package will be created in test_dir, and test_dir will be
# removed after the tests again.
# ModuleFinder searches in a path that contains test_dir, plus
# the standard path search directory.


def _do_test(
    test_dir,
    import_this,
    modules,
    missing,  # noqa: ARG001
    maybe_missing,  # noqa: ARG001
    source,
    report=False,
    debug=0,  # noqa: ARG001
    modulefinder_class=ModuleFinder,
    **kwargs,
):
    create_package(test_dir, source)
    finder = modulefinder_class(
        ConstantsModule(),
        path=[test_dir, *sys.path],
        # debug=debug,
        **kwargs,
    )
    finder.modules = []  # remove default modules
    finder._modules = {}  # pylint: disable=protected-access
    finder.include_module(import_this)
    if report:
        finder.report_missing_modules()
    modules = sorted(set(modules))
    found = sorted([module.name for module in finder.modules])
    # check if we found what we expected, not more, not less
    assert found == modules

    # check for missing and maybe missing modules
    # bad, maybe = finder.any_missing_maybe()
    # self.assertEqual(bad, missing)
    # self.assertEqual(maybe, maybe_missing)


@pytest.mark.parametrize(
    ("import_this", "modules", "missing", "maybe_missing", "source"),
    [
        absolute_import_test,
        coding_default_utf8_test,
        coding_explicit_cp1252_test,
        coding_explicit_utf8_test,
        extended_opargs_test,
        maybe_test,
        maybe_test_new,
        package_test,
        relative_import_test,
        relative_import_test_2,
        relative_import_test_3,
        relative_import_test_4,
        same_name_as_bad_test,
        sub_package_test,
    ],
    ids=[
        "absolute_import_test",
        "coding_default_utf8_test",
        "coding_explicit_cp1252_test",
        "coding_explicit_utf8_test",
        "extended_opargs_test",
        "maybe_test",
        "maybe_test_new",
        "package_test",
        "relative_import_test",
        "relative_import_test_2",
        "relative_import_test_3",
        "relative_import_test_4",
        "same_name_as_bad_test",
        "sub_package_test",
    ],
)
def test_finder(
    tmp_path,
    import_this,
    modules,
    missing,
    maybe_missing,
    source,
):
    """Provides test cases for ModuleFinder class."""
    _do_test(tmp_path, import_this, modules, missing, maybe_missing, source)


def test_bytecode(tmp_path):
    """Provides bytecode test case for ModuleFinder class."""
    base_path = tmp_path / "a"
    source_path = base_path.with_suffix(SOURCE_SUFFIXES[0])
    bytecode_path = base_path.with_suffix(BYTECODE_SUFFIXES[0])
    with source_path.open("wb") as file:
        file.write(b"testing_modulefinder = True\n")
    py_compile.compile(os.fspath(source_path), cfile=os.fspath(bytecode_path))
    os.remove(source_path)
    _do_test(tmp_path, *bytecode_test)


def test_zip_include_packages(tmp_path):
    """Provides test cases for ModuleFinder class."""
    _do_test(
        tmp_path,
        *sub_package_test,
        zip_exclude_packages=["*"],
        zip_include_packages=["p"],
    )


def test_zip_exclude_packages(tmp_path):
    """Provides test cases for ModuleFinder class."""
    _do_test(
        tmp_path,
        *sub_package_test,
        zip_exclude_packages=["p"],
        zip_include_packages=["*"],
    )



from __future__ import annotations

import os
import sys
from tkinter import Button, Label, PhotoImage, TclVersion, Tk


def _test():
    root = Tk()
    text = (
        f"This is Tcl/Tk version {TclVersion}\n"
        "This should be a cedilla: \xe7"
    )
    label = Label(root, text=text)
    label.pack()
    test = Button(
        root,
        text="Click me!",
        command=lambda root=root: root.test.configure(
            text="[%s]" % root.test["text"]
        ),
    )
    datadir = (
        os.path.dirname(sys.executable)
        if getattr(sys, "frozen", False)
        else ".."
    )
    icon = PhotoImage(file=os.path.join(datadir, "icon", "favicon.png"))
    root.tk.call("wm", "iconphoto", root._w, icon)
    test.pack()
    root.test = test
    quit = Button(root, text="QUIT", command=root.destroy)
    quit.pack()
    # The following three commands are needed so the window pops
    # up on top on Windows...
    root.iconify()
    root.update()
    root.deiconify()
    root.mainloop()


if __name__ == "__main__":
    _test()



"""A collection of functions which are triggered automatically by finder when
glib package is included.
"""

from __future__ import annotations

from ..finder import ModuleFinder
from ..module import Module


def load_glib(finder: ModuleFinder, module: Module) -> None:  # noqa: ARG001
    """Ignore globals that are imported."""
    module.global_names.update(
        [
            "GError",
            "IOChannel",
            "IO_ERR",
            "IO_FLAG_APPEND",
            "IO_FLAG_GET_MASK",
            "IO_FLAG_IS_READABLE",
            "IO_FLAG_IS_SEEKABLE",
            "IO_FLAG_IS_WRITEABLE",
            "IO_FLAG_MASK",
            "IO_FLAG_NONBLOCK",
            "IO_FLAG_SET_MASK",
            "IO_HUP",
            "IO_IN",
            "IO_NVAL",
            "IO_OUT",
            "IO_PRI",
            "IO_STATUS_AGAIN",
            "IO_STATUS_EOF",
            "IO_STATUS_ERROR",
            "IO_STATUS_NORMAL",
            "Idle",
            "MainContext",
            "MainLoop",
            "OPTION_ERROR",
            "OPTION_ERROR_BAD_VALUE",
            "OPTION_ERROR_FAILED",
            "OPTION_ERROR_UNKNOWN_OPTION",
            "OPTION_FLAG_FILENAME",
            "OPTION_FLAG_HIDDEN",
            "OPTION_FLAG_IN_MAIN",
            "OPTION_FLAG_NOALIAS",
            "OPTION_FLAG_NO_ARG",
            "OPTION_FLAG_OPTIONAL_ARG",
            "OPTION_FLAG_REVERSE",
            "OPTION_REMAINING",
            "OptionContext",
            "OptionGroup",
            "PRIORITY_DEFAULT",
            "PRIORITY_DEFAULT_IDLE",
            "PRIORITY_HIGH",
            "PRIORITY_HIGH_IDLE",
            "PRIORITY_LOW",
            "Pid",
            "PollFD",
            "SPAWN_CHILD_INHERITS_STDIN",
            "SPAWN_DO_NOT_REAP_CHILD",
            "SPAWN_FILE_AND_ARGV_ZERO",
            "SPAWN_LEAVE_DESCRIPTORS_OPEN",
            "SPAWN_SEARCH_PATH",
            "SPAWN_STDERR_TO_DEV_NULL",
            "SPAWN_STDOUT_TO_DEV_NULL",
            "Source",
            "Timeout",
            "child_watch_add",
            "filename_display_basename",
            "filename_display_name",
            "filename_from_utf8",
            "get_application_name",
            "get_current_time",
            "get_prgname",
            "glib_version",
            "idle_add",
            "io_add_watch",
            "main_context_default",
            "main_depth",
            "markup_escape_text",
            "set_application_name",
            "set_prgname",
            "source_remove",
            "spawn_async",
            "timeout_add",
            "timeout_add_seconds",
        ]
    )



from __future__ import annotations

import moda  # noqa: F401
from modb import b  # noqa: F401

from . import modc  # noqa: F401
from .modd import d  # noqa: F401

from mode import *  # noqa: F403 isort:skip
from ..modf import *  # noqa: F403 isort:skip
import modg.submod  # noqa: F401 isort:skip

try:
    pass
finally:
    import modh  # noqa: F401



"""A collection of functions which are triggered automatically by finder when
pytz package is included.
"""

from __future__ import annotations

import os
from pathlib import Path

from ..finder import ModuleFinder
from ..module import Module


def load_pytz(finder: ModuleFinder, module: Module) -> None:
    """The pytz module requires timezone data to be found in a known directory
    or in the zip file where the package is written.
    """
    target_path = Path("lib", "pytz", "zoneinfo")
    data_path = module.path[0] / "zoneinfo"
    if not data_path.is_dir():
        # Fedora (and possibly other systems) use a separate location to
        # store timezone data so look for that here as well
        pytz = __import__("pytz")
        data_path = Path(
            getattr(pytz, "_tzinfo_dir", None)
            or os.getenv("PYTZ_TZDATADIR")
            or "/usr/share/zoneinfo"
        )
        if data_path.is_dir():
            finder.add_constant("PYTZ_TZDATADIR", os.fspath(target_path))
    if data_path.is_dir():
        if module.in_file_system >= 1:
            finder.include_files(
                data_path, target_path, copy_dependent_files=False
            )
        else:
            finder.zip_include_files(data_path, Path("pytz", "zoneinfo"))



"""A simple script to demonstrate opencv-python."""

from __future__ import annotations

import os
import sys

sys.OpenCV_LOADER_DEBUG = True
import cv2 as cv  # noqa


def find_data_file(filename):
    if getattr(sys, "frozen", False):
        # The application is frozen
        datadir = os.path.dirname(sys.executable)
    else:
        # The application is not frozen
        # Change this bit to match where you store your data files:
        datadir = os.path.dirname(__file__)
    return os.path.join(datadir, filename)


img = cv.imread(find_data_file("image.png"))
if img is None:
    sys.exit("Could not read the image.")
cv.imshow("Display window", img)
k = cv.waitKey(15000)
if k == ord("s"):
    cv.imwrite("image1.png", img)



"""Tests for cx_Freeze.command.bdist_msi."""

from __future__ import annotations

import sys
from pathlib import Path
from sysconfig import get_platform

import pytest
from setuptools import Distribution

from cx_Freeze.sandbox import run_setup

if sys.platform == "win32":
    from cx_Freeze.command.bdist_msi import BdistMSI

DIST_ATTRS = {
    "name": "foo",
    "version": "0.0",
    "executables": [],
    "script_name": "setup.py",
}
FIXTURE_DIR = Path(__file__).resolve().parent


@pytest.mark.skipif(sys.platform != "win32", reason="Windows tests")
@pytest.mark.datafiles(FIXTURE_DIR.parent / "samples" / "msi_binary_data")
def test_bdist_msi(datafiles: Path):
    """Test the msi_binary_data sample."""
    run_setup(datafiles / "setup.py", ["bdist_msi"])
    platform = get_platform().replace("win-amd64", "win64")
    file_created = datafiles / "dist" / f"hello-0.1-{platform}.msi"
    assert file_created.is_file()


@pytest.mark.skipif(sys.platform != "win32", reason="Windows tests")
def test_bdist_msi_target_name():
    """Test the bdist_msi with extra target_name option."""
    dist = Distribution(DIST_ATTRS)
    cmd = BdistMSI(dist)
    cmd.target_name = "mytest"
    cmd.finalize_options()
    cmd.ensure_finalized()
    assert cmd.fullname == "mytest-0.0"


@pytest.mark.skipif(sys.platform != "win32", reason="Windows tests")
def test_bdist_msi_target_name_and_version():
    """Test the bdist_msi with extra target options."""
    dist = Distribution(DIST_ATTRS)
    cmd = BdistMSI(dist)
    cmd.target_name = "mytest"
    cmd.target_version = "0.1"
    cmd.finalize_options()
    cmd.ensure_finalized()
    assert cmd.fullname == "mytest-0.1"


@pytest.mark.skipif(sys.platform != "win32", reason="Windows tests")
@pytest.mark.datafiles(FIXTURE_DIR.parent / "samples" / "msi_binary_data")
def test_bdist_msi_target_name_with_extension(datafiles: Path):
    """Test the msi_binary_data sample, with a specified target_name that
    includes an ".msi" extension.
    """
    msi_name = "output.msi"
    run_setup(datafiles / "setup.py", ["bdist_msi", "--target-name", msi_name])
    file_created = datafiles / "dist" / msi_name
    assert file_created.is_file()



"""Tests for cx_Freeze.command.build_exe."""

from __future__ import annotations

import os
import subprocess
import sys
from pathlib import Path
from sysconfig import get_platform, get_python_version

import pytest
from generate_samples import create_package, sub_package_test

from cx_Freeze.sandbox import run_setup

PLATFORM = get_platform()
PYTHON_VERSION = get_python_version()
BUILD_EXE_DIR = f"build/exe.{PLATFORM}-{PYTHON_VERSION}"
FIXTURE_DIR = Path(__file__).resolve().parent

OUTPUT1 = "Hello from cx_Freeze Advanced #1\nTest freeze module #1\n"
OUTPUT2 = "Hello from cx_Freeze Advanced #2\nTest freeze module #2\n"

OUTPUT_SUBPACKAGE_TEST = "This is p.p1\nThis is p.q.q1\n"


@pytest.mark.datafiles(FIXTURE_DIR.parent / "samples" / "advanced")
def test_build_exe_advanced(datafiles: Path):
    """Test the advanced sample."""
    run_setup(
        datafiles / "setup.py", ["build_exe", "--silent", "--excludes=tkinter"]
    )
    suffix = ".exe" if sys.platform == "win32" else ""

    executable = datafiles / BUILD_EXE_DIR / f"advanced_1{suffix}"
    assert executable.is_file()
    output = subprocess.check_output(
        [os.fspath(executable)], text=True, timeout=10
    )
    assert output == OUTPUT1

    executable = datafiles / BUILD_EXE_DIR / f"advanced_2{suffix}"
    assert executable.is_file()
    output = subprocess.check_output(
        [os.fspath(executable)], text=True, timeout=10
    )
    assert output == OUTPUT2


@pytest.mark.datafiles(FIXTURE_DIR.parent / "samples" / "asmodule")
def test_build_exe_asmodule(datafiles: Path):
    """Test the asmodule sample."""
    run_setup(
        datafiles / "setup.py", ["build_exe", "--silent", "--excludes=tkinter"]
    )
    suffix = ".exe" if sys.platform == "win32" else ""
    executable = datafiles / BUILD_EXE_DIR / f"asmodule{suffix}"
    assert executable.is_file()
    output = subprocess.check_output(
        [os.fspath(executable)], text=True, timeout=10
    )
    assert output.startswith("Hello from cx_Freeze")


@pytest.mark.datafiles(FIXTURE_DIR.parent / "samples" / "simple")
def test_build_exe_simple(datafiles: Path):
    """Test the simple sample."""
    run_setup(
        datafiles / "setup.py", ["build_exe", "--silent", "--excludes=tkinter"]
    )
    suffix = ".exe" if sys.platform == "win32" else ""
    executable = datafiles / BUILD_EXE_DIR / f"hello{suffix}"
    assert executable.is_file()
    output = subprocess.check_output(
        [os.fspath(executable)], text=True, timeout=10
    )
    assert output.startswith("Hello from cx_Freeze")


@pytest.mark.datafiles(FIXTURE_DIR.parent / "samples" / "sqlite")
def test_build_exe_sqlite(datafiles: Path):
    """Test the sqlite sample."""
    run_setup(
        datafiles / "setup.py", ["build_exe", "--silent", "--excludes=tkinter"]
    )
    suffix = ".exe" if sys.platform == "win32" else ""
    executable = datafiles / BUILD_EXE_DIR / f"test_sqlite3{suffix}"
    assert executable.is_file()
    output = subprocess.check_output(
        [os.fspath(executable)], text=True, timeout=10
    )
    assert output.startswith("dump.sql created")


def test_zip_include_packages(tmp_path):
    """Provides test cases for ModuleFinder class."""
    source = sub_package_test[4]
    create_package(tmp_path, source)
    run_setup(
        tmp_path / "setup.py",
        [
            "build_exe",
            "--silent",
            "--excludes=tkinter",
            "--zip-exclude-packages=*",
            "--zip-include-packages=p",
        ],
    )
    suffix = ".exe" if sys.platform == "win32" else ""
    executable = tmp_path / BUILD_EXE_DIR / f"main{suffix}"
    assert executable.is_file()
    output = subprocess.check_output(
        [os.fspath(executable)], text=True, timeout=10
    )
    assert output == OUTPUT_SUBPACKAGE_TEST


def test_zip_exclude_packages(tmp_path):
    """Provides test cases for ModuleFinder class."""
    source = sub_package_test[4]
    create_package(tmp_path, source)
    run_setup(
        tmp_path / "setup.py",
        [
            "build_exe",
            "--silent",
            "--excludes=tkinter",
            "--zip-exclude-packages=p",
            "--zip-include-packages=*",
        ],
    )
    suffix = ".exe" if sys.platform == "win32" else ""
    executable = tmp_path / BUILD_EXE_DIR / f"main{suffix}"
    assert executable.is_file()
    output = subprocess.check_output(
        [os.fspath(executable)], text=True, timeout=10
    )
    assert output == OUTPUT_SUBPACKAGE_TEST



# A setup script to demonstrate the use of complex importlib machinery at work
#
# Each of these samples (server_simple.py, web_srv.py and wsgiserver.py) are
# acquired from GitHub via the get_examples.py script and stored for
# convenience.
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the script without Python

from __future__ import annotations

from cx_Freeze import Executable, setup

buildOptions = dict(zip_include_packages=["*"], zip_exclude_packages=[])
executables = [
    Executable("server_simple.py"),
    Executable("web_srv.py"),
    Executable("wsgiserver.py"),
]

setup(
    name="test_importlib",
    version="0.1",
    description="cx_Freeze script for web servers that test importlib",
    executables=executables,
    options=dict(build_exe=buildOptions),
)



"""Test winversioninfo."""

from __future__ import annotations

import sys

import pytest

from cx_Freeze.winversioninfo import Version, VersionInfo


@pytest.mark.skipif(sys.platform != "win32", reason="Windows tests")
class TestVersionInfo:
    """Test VersionInfo class."""

    def test___init__(self):
        """Tests the default value for the VersionInfo class."""
        input_version = "9.9.9.9"
        default_version = VersionInfo(input_version)
        assert default_version.version == Version(input_version)
        assert default_version.internal_name is None
        assert default_version.original_filename is None
        assert default_version.comments is None
        assert default_version.company is None
        assert default_version.description is None
        assert default_version.copyright is None
        assert default_version.trademarks is None
        assert default_version.product is None
        assert default_version.dll is None
        assert default_version.debug is None
        assert default_version.verbose is True

    def test___init__pads_short_versions(self):
        """Tests that short versions get padded to the expected x4 digit
        windows versions.
        """
        input_version = "9"
        default_version = VersionInfo(input_version)
        assert (
            default_version.version.base_version
            == Version(input_version).base_version
        )

    def test___init__with_kwargs(self):
        """Tests keyword values for the VersionInfo class."""
        input_version = "9.9.9.9"
        input_internal_name = "Test Internal Name"
        input_original_filename = "TestTempFileName"
        input_comments = "TestComment"
        input_company = "TestCompany"
        input_description = "TestDescription"
        input_copyright = "TestCopyright"
        input_trademarks = "TestMark"
        input_product = object()
        input_dll = True
        input_debug = True
        input_verbose = False

        version_instance = VersionInfo(
            version=input_version,
            internal_name=input_internal_name,
            original_filename=input_original_filename,
            comments=input_comments,
            company=input_company,
            description=input_description,
            copyright=input_copyright,
            trademarks=input_trademarks,
            product=input_product,
            dll=input_dll,
            debug=input_debug,
            verbose=input_verbose,
        )

        assert version_instance.version == Version(input_version)
        assert version_instance.internal_name == input_internal_name
        assert version_instance.original_filename == input_original_filename
        assert version_instance.comments == input_comments
        assert version_instance.company == input_company
        assert version_instance.description == input_description
        assert version_instance.copyright == input_copyright
        assert version_instance.trademarks == input_trademarks
        assert version_instance.product == input_product
        assert version_instance.dll is input_dll
        assert version_instance.debug is input_debug
        assert version_instance.verbose is input_verbose



"""
Sample to test the 'pycountry' hook

"""

from __future__ import annotations

import gettext

import pycountry

print("countries:", len(pycountry.countries))
print("BR-FN", pycountry.subdivisions.get(code="BR-FN"))

name = gettext.translation("iso3166", pycountry.LOCALES_DIR, languages=["pt"])
name.install()
_ = name.gettext
print("Brazil ->", _("Brazil"))



"""A simple setup script to create executables using scipy."""

# test_scipy.py is a very simple scipy application that demonstrates
# its use.
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the application

from cx_Freeze import Executable, setup


executables = [
    Executable("test_scipy.py"),
]

setup(
    name="scipy_samples",
    version="0.1",
    description="Sample scipy script",
    executables=executables,
)



"""Setuptools script for cx_Freeze.

Use ONE of the following commands to install from source:
    pip install .
    python setup.py build install

Use the following commands to install in the development mode:
    pip install -r requirements-dev.txt
    pip install -e . --no-build-isolation --no-deps

"""
from __future__ import annotations

# pylint: disable=attribute-defined-outside-init
import os
import sys
from pathlib import Path
from shutil import which
from subprocess import CalledProcessError, check_call, check_output
from sysconfig import get_config_var, get_platform, get_python_version

import setuptools.command.build_ext
from setuptools import Extension, setup
from setuptools.errors import LinkError

ENABLE_SHARED = bool(get_config_var("Py_ENABLE_SHARED"))
PLATFORM = get_platform()
IS_MACOS = PLATFORM.startswith("macos")
IS_MINGW = PLATFORM.startswith("mingw")
IS_WINDOWS = PLATFORM.startswith("win")
IS_CONDA = Path(sys.prefix, "conda-meta").is_dir()


class BuildBases(setuptools.command.build_ext.build_ext):
    """Build C bases and extension."""

    def build_extension(self, ext):
        if "bases" not in ext.name:
            super().build_extension(ext)
            return
        if IS_MINGW or IS_WINDOWS and self.compiler.compiler_type == "mingw32":
            ext.sources.append("source/bases/manifest.rc")
        objects = self.compiler.compile(
            ext.sources,
            output_dir=self.build_temp,
            include_dirs=ext.include_dirs,
            debug=self.debug,
            depends=ext.depends,
        )
        filename = Path(self.get_ext_filename(ext.name)).with_suffix("")
        fullname = os.fspath(Path(self.build_lib, filename))
        library_dirs = ext.library_dirs or []
        libraries = self.get_libraries(ext)
        extra_args = ext.extra_link_args or []
        if IS_MINGW or IS_WINDOWS:
            compiler_type = self.compiler.compiler_type
            # support for delay load [windows]
            for arg in extra_args[:]:
                if arg.startswith("/DELAYLOAD:"):
                    lib_name = arg[len("/DELAYLOAD:") :]
                    extra_args.remove(arg)
                    dll_path = self._get_dll_path(lib_name)
                    dll_name = dll_path.name
                    if compiler_type == "msvc":
                        extra_args.append(f"/DELAYLOAD:{dll_name}")
                        if lib_name not in libraries:
                            libraries.append(lib_name)
                        if "delayimp" not in libraries:
                            libraries.append("delayimp")
                    elif compiler_type == "mingw32":
                        if lib_name in libraries:
                            libraries.remove(lib_name)
                        lib_dir, library = self._dlltool_delay_load(lib_name)
                        for linker_option in self.compiler.linker_exe:
                            if "clang" in linker_option:
                                extra_args.append(f"-Wl,-delayload,{dll_name}")
                                break
                        if PLATFORM.startswith("mingw_i686"):  # mingw32
                            # disable delay load to avoid a Segmentation fault
                            libraries.append(lib_name)
                        else:
                            libraries.append(library)
                        library_dirs.append(lib_dir)
            if compiler_type == "msvc":
                extra_args.append("/MANIFEST")
            elif compiler_type == "mingw32":
                if "Win32GUI" in ext.name:
                    extra_args.append("-mwindows")
                else:
                    extra_args.append("-mconsole")
                extra_args.append("-municode")
        else:
            library_dirs.append(get_config_var("LIBPL"))
            if not ENABLE_SHARED:
                library_dirs.append(get_config_var("LIBDIR"))
            abiflags = get_config_var("abiflags")
            libraries.append(f"python{get_python_version()}{abiflags}")
            if get_config_var("LINKFORSHARED") and not IS_MACOS:
                extra_args.extend(get_config_var("LINKFORSHARED").split())
            if get_config_var("LIBS"):
                extra_args.extend(get_config_var("LIBS").split())
            if get_config_var("LIBM"):
                extra_args.append(get_config_var("LIBM"))
            if get_config_var("BASEMODLIBS"):
                extra_args.extend(get_config_var("BASEMODLIBS").split())
            if get_config_var("LOCALMODLIBS"):
                extra_args.extend(get_config_var("LOCALMODLIBS").split())
            if IS_MACOS:
                # macOS on Github Actions
                extra_args.append("-Wl,-export_dynamic")
            else:
                if not self.debug:
                    extra_args.append("-s")
                extra_args.append("-Wl,-rpath,$ORIGIN/lib")
                extra_args.append("-Wl,-rpath,$ORIGIN/../lib")
        for arg in (None, "--no-lto"):
            if arg:
                extra_args.append(arg)
            try:
                self.compiler.link_executable(
                    objects,
                    fullname,
                    libraries=libraries,
                    library_dirs=library_dirs,
                    runtime_library_dirs=ext.runtime_library_dirs,
                    extra_postargs=extra_args,
                    debug=self.debug,
                )
            except LinkError:
                if IS_MINGW or IS_WINDOWS:
                    raise
                continue
            else:
                break

    def get_ext_filename(self, fullname):
        if fullname.endswith("util"):
            return super().get_ext_filename(fullname)
        # Examples of returned names:
        # console-cp37-win32.exe, console-cp39-win_amd64.exe,
        # console-cpython-39-x86_64-linux-gnu, console-cpython-36m-darwin
        ext_path = Path(*fullname.split("."))
        name = ext_path.name
        if IS_MINGW or IS_WINDOWS:
            platform_nodot = PLATFORM.replace(".", "").replace("-", "_")
            soabi = f"{sys.implementation.cache_tag}-{platform_nodot}"
            suffix = ".exe"
        else:
            soabi = get_config_var("SOABI")
            suffix = ""
        name_base = f"{name}-{soabi}"
        return os.fspath(ext_path.parent / (name_base + suffix))

    @staticmethod
    def _get_dll_path(name: str) -> Path:
        """Find the dll by name, priority by pyd extension."""
        pattern_pyd = f"{name}*.pyd"
        pattern_dll = f"{name}*.dll"
        for path in map(Path, sys.path):
            if not path.is_dir():
                continue
            for dll_path in path.glob(pattern_pyd):
                return dll_path.resolve()
            for dll_path in path.glob(pattern_dll):
                return dll_path.resolve()
        return Path(f"{name}.dll")

    def _dlltool_delay_load(self, name: str) -> tuple[str, str]:
        """Get the delay load library to use with mingw32 compilers."""
        dir_name = f"libdl.{PLATFORM}-{get_python_version()}"
        library_dir = Path(self.build_temp, dir_name)
        library_dir.mkdir(parents=True, exist_ok=True)
        # Use gendef and dlltool to generate the library (.a and .delay.a)
        dll_path = self._get_dll_path(name)
        gendef_exe = Path(which("gendef"))
        def_data = check_output(_make_strs([gendef_exe, "-", dll_path]))
        def_name = library_dir / f"{name}.def"
        def_name.write_bytes(def_data)
        lib_path = library_dir / f"lib{name}.a"
        library = f"{name}.delay"
        dlb_path = library_dir / f"lib{library}.a"
        dlltool_exe = gendef_exe.parent / "dlltool.exe"
        dlltool = [dlltool_exe, "-d", def_name, "-D", dll_path, "-l", lib_path]
        output_delaylib_args = ["-y", dlb_path]
        try:
            # GNU binutils dlltool support --output-delaylib
            check_call(_make_strs(dlltool + output_delaylib_args))
        except CalledProcessError:
            # LLVM dlltool only supports generating an import library
            check_call(_make_strs(dlltool))
            library = name
        return os.fspath(library_dir), library

    def _copy_libraries_to_bases(self):
        """Copy standard libraries to cx_Freeze wheel, on posix systems, when
        python is compiled with --disable-shared, as is done in manylinux and
        macpython. Modules such as math, _struct and zlib, which are normally
        embedded in python, are compiled separately.
        Also, copies tcl/tk libraries.
        """
        if IS_MINGW or IS_WINDOWS or IS_CONDA or ENABLE_SHARED:
            return
        bases = f"{self.build_lib}/cx_Freeze/bases"
        if bool(get_config_var("DESTSHARED")):
            source_path = Path(get_config_var("DESTSHARED"))
            target_path = f"{bases}/lib-dynload"
            self.mkpath(target_path)
            for source in source_path.iterdir():
                self.copy_file(source.as_posix(), target_path)
        # tcl/tk are detected in /usr/local/lib or /usr/share
        try:
            tkinter = __import__("tkinter")
        except (ImportError, AttributeError):
            return
        root = tkinter.Tk(useTk=False)
        tcl_library = Path(root.tk.exprstring("$tcl_library"))
        # source paths of tcl8.6, tcl8 and tk8.6
        source_paths = [
            tcl_library,
            tcl_library.with_suffix(""),
            tcl_library.parent / tcl_library.name.replace("tcl", "tk"),
        ]
        for source_path in source_paths:
            target_path = f"{bases}/tcltk/{source_path.name}"
            self.mkpath(target_path)
            for source in source_path.rglob("*"):
                target = os.fspath(
                    target_path / source.relative_to(source_path)
                )
                if source.is_dir():
                    self.mkpath(target)
                else:
                    self.copy_file(source.as_posix(), target)

    def run(self):
        self._copy_libraries_to_bases()
        super().run()


def _make_strs(paths: list[str | Path]) -> list[str]:
    """Convert paths to strings for legacy compatibility."""
    if sys.version_info > (3, 8) and not (IS_MINGW or IS_WINDOWS):
        return paths
    return list(map(os.fspath, paths))


def get_extensions():
    """Build base executables and util module extension."""
    # [Windows only] With binaries included in bases, the compilation is
    # optional in the development mode.
    depends = ["source/bases/common.c"]
    optional = IS_WINDOWS and os.environ.get("CIBUILDWHEEL", "0") != "1"
    extensions = [
        Extension(
            "cx_Freeze.bases.console",
            ["source/bases/console.c"],
            depends=depends,
            optional=optional,
        )
    ]

    if IS_MINGW or IS_WINDOWS:
        extensions += [
            Extension(
                "cx_Freeze.bases.Win32GUI",
                ["source/bases/Win32GUI.c"],
                depends=depends,
                libraries=["user32"],
                optional=optional,
            ),
            Extension(
                "cx_Freeze.bases.Win32Service",
                ["source/bases/Win32Service.c"],
                depends=depends,
                extra_link_args=["/DELAYLOAD:cx_Logging"],
                libraries=["advapi32"],
                optional=optional,
            ),
            Extension(
                "cx_Freeze.util",
                ["source/util.c"],
                libraries=["imagehlp", "shlwapi"],
                optional=optional,
            ),
        ]
    return extensions


if __name__ == "__main__":
    setup(
        cmdclass={"build_ext": BuildBases},
        options={"install": {"optimize": 1}},
        ext_modules=get_extensions(),
    )



"""A simple setup script to create executable using matplotlib
   This version requires the mpl-data in the zip file."""

from __future__ import annotations

# test_matplotlib_tk.py is a very simple matplotlib application that
# demonstrates its use.
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the application
import os
import sys
import sysconfig

from cx_Freeze import Executable, setup

base = "Win32GUI" if sys.platform == "win32" else None

platform = sysconfig.get_platform()
python_version = sysconfig.get_python_version()
dir_name = f"zip.{platform}-{python_version}"
build_exe_dir = os.path.join("build", dir_name)

options = {
    "build_exe": {
        # Sometimes a little fine-tuning is needed
        # exclude all backends except tkinter
        "excludes": [
            "gi",
            "gtk",
            "PyQt4",
            "PyQt5",
            "PyQt6",
            "PySide2",
            "PySide6",
            "shiboken2",
            "shiboken6",
            "wx",
        ],
        "zip_include_packages": ["*"],
        "zip_exclude_packages": [],
        "build_exe": build_exe_dir,
    }
}

executables = [
    Executable("test_matplotlib_tk.py", base=base),
]

setup(
    name="matplotlib_samples",
    version="0.1",
    description="Sample matplotlib script",
    executables=executables,
    options=options,
)



from __future__ import annotations

import clr
import System  # noqa

clr.AddReference("System.Windows.Forms")
import System.Windows.Forms as WinForms  # noqa
from System.IO import File  # noqa
from System.Text import Encoding  # noqa
from System.Threading import ApartmentState, Thread, ThreadStart  # noqa


class Wordpad(WinForms.Form):
    """A simple example winforms application similar to wordpad."""

    def __init__(self):
        super().__init__()
        self.filename = ""
        self.word_wrap = True
        self.doctype = 1
        self.InitializeComponent()
        self.NewDocument()

    def InitializeComponent(self):
        """Initialize form components."""
        self.components = System.ComponentModel.Container()

        self.openFileDialog = WinForms.OpenFileDialog()
        self.saveFileDialog = WinForms.SaveFileDialog()

        self.mainMenu = WinForms.MainMenu()

        self.fileMenu = WinForms.MenuItem()
        self.menuFileNew = WinForms.MenuItem()
        self.menuFileOpen = WinForms.MenuItem()
        self.menuFileSave = WinForms.MenuItem()
        self.menuFileSaveAs = WinForms.MenuItem()
        self.menuFileSep_1 = WinForms.MenuItem()
        self.menuFileExit = WinForms.MenuItem()

        self.editMenu = WinForms.MenuItem()
        self.menuEditUndo = WinForms.MenuItem()
        self.menuEditRedo = WinForms.MenuItem()
        self.menuEditSep_1 = WinForms.MenuItem()
        self.menuEditCut = WinForms.MenuItem()
        self.menuEditCopy = WinForms.MenuItem()
        self.menuEditPaste = WinForms.MenuItem()
        self.menuEditSep_2 = WinForms.MenuItem()
        self.menuEditSelectAll = WinForms.MenuItem()

        self.formatMenu = WinForms.MenuItem()
        self.menuFormatFont = WinForms.MenuItem()
        self.menuFormatWordWrap = WinForms.MenuItem()

        self.aboutMenu = WinForms.MenuItem()
        self.menuHelpAbout = WinForms.MenuItem()

        self.richTextBox = WinForms.RichTextBox()
        self.statusBarPanel1 = WinForms.StatusBarPanel()
        self.statusBar = WinForms.StatusBar()
        self.fontDialog = WinForms.FontDialog()
        self.statusBarPanel1.BeginInit()

        # ===================================================================
        # File Menu
        # ===================================================================

        self.menuFileNew.Text = "&New"
        self.menuFileNew.Shortcut = WinForms.Shortcut.CtrlN
        self.menuFileNew.ShowShortcut = False
        self.menuFileNew.Index = 0
        self.menuFileNew.Click += self.OnClickFileNew

        self.menuFileOpen.Text = "&Open"
        self.menuFileOpen.Shortcut = WinForms.Shortcut.CtrlO
        self.menuFileOpen.ShowShortcut = False
        self.menuFileOpen.Index = 1
        self.menuFileOpen.Click += self.OnClickFileOpen

        self.menuFileSave.Text = "&Save"
        self.menuFileSave.Shortcut = WinForms.Shortcut.CtrlS
        self.menuFileSave.ShowShortcut = False
        self.menuFileSave.Index = 2
        self.menuFileSave.Click += self.OnClickFileSave

        self.menuFileSaveAs.Text = "Save &As"
        self.menuFileSaveAs.Index = 3
        self.menuFileSaveAs.Click += self.OnClickFileSaveAs

        self.menuFileSep_1.Text = "-"
        self.menuFileSep_1.Index = 4

        self.menuFileExit.Text = "E&xit"
        self.menuFileExit.Shortcut = WinForms.Shortcut.AltF4
        self.menuFileExit.ShowShortcut = False
        self.menuFileExit.Index = 5
        self.menuFileExit.Click += self.OnClickFileExit

        self.fileMenu.Text = "&File"
        self.fileMenu.Index = 0

        items = (
            self.menuFileNew,
            self.menuFileOpen,
            self.menuFileSave,
            self.menuFileSaveAs,
            self.menuFileSep_1,
            self.menuFileExit,
        )

        self.fileMenu.MenuItems.AddRange(items)

        # ===================================================================
        # Edit menu
        # ===================================================================

        self.menuEditUndo.Text = "&Undo"
        self.menuEditUndo.Shortcut = WinForms.Shortcut.CtrlZ
        self.menuEditUndo.Index = 0
        self.menuEditUndo.Click += self.OnClickEditUndo

        self.menuEditRedo.Text = "&Redo"
        self.menuEditRedo.Shortcut = WinForms.Shortcut.CtrlY
        self.menuEditRedo.Index = 1
        self.menuEditRedo.Click += self.OnClickEditRedo

        self.menuEditSep_1.Text = "-"
        self.menuEditSep_1.Index = 2

        self.menuEditCut.Text = "Cut"
        self.menuEditCut.Shortcut = WinForms.Shortcut.CtrlX
        self.menuEditCut.Index = 3
        self.menuEditCut.Click += self.OnClickEditCut

        self.menuEditCopy.Text = "Copy"
        self.menuEditCopy.Shortcut = WinForms.Shortcut.CtrlC
        self.menuEditCopy.Index = 4
        self.menuEditCopy.Click += self.OnClickEditCopy

        self.menuEditPaste.Text = "Paste"
        self.menuEditPaste.Shortcut = WinForms.Shortcut.CtrlV
        self.menuEditPaste.Index = 5
        self.menuEditPaste.Click += self.OnClickEditPaste

        self.menuEditSelectAll.Text = "Select All"
        self.menuEditSelectAll.Shortcut = WinForms.Shortcut.CtrlA
        self.menuEditSelectAll.Index = 7
        self.menuEditSelectAll.Click += self.OnClickEditSelectAll

        self.menuEditSep_2.Text = "-"
        self.menuEditSep_2.Index = 6

        self.editMenu.Text = "&Edit"
        self.editMenu.Index = 1

        items = (
            self.menuEditUndo,
            self.menuEditRedo,
            self.menuEditSep_1,
            self.menuEditCut,
            self.menuEditCopy,
            self.menuEditPaste,
            self.menuEditSep_2,
            self.menuEditSelectAll,
        )

        self.editMenu.MenuItems.AddRange(items)

        # ===================================================================
        # Format Menu
        # ===================================================================

        self.menuFormatWordWrap.Text = "Word Wrap"
        self.menuFormatWordWrap.Checked = self.word_wrap
        self.menuFormatWordWrap.Index = 1
        self.menuFormatWordWrap.Click += self.OnClickFormatWordWrap

        self.menuFormatFont.Text = "Fo&nt"
        self.menuFormatFont.Index = 0
        self.menuFormatFont.Click += self.OnClickFormatFont

        self.formatMenu.Text = "F&ormat"
        self.formatMenu.Index = 2

        items = (self.menuFormatWordWrap, self.menuFormatFont)

        self.formatMenu.MenuItems.AddRange(items)

        # ===================================================================
        # About menu
        # ===================================================================

        self.menuHelpAbout.Text = "&About"
        self.menuHelpAbout.Index = 0
        self.menuHelpAbout.Click += self.OnClickHelpAbout

        self.aboutMenu.Text = "&Help"
        self.aboutMenu.Index = 3
        self.aboutMenu.MenuItems.Add(self.menuHelpAbout)

        self.statusBarPanel1.Dock = WinForms.DockStyle.Fill
        self.statusBarPanel1.Text = "Ready"
        self.statusBarPanel1.Width = 755

        self.richTextBox.Dock = WinForms.DockStyle.Fill
        self.richTextBox.Size = System.Drawing.Size(795, 485)
        self.richTextBox.TabIndex = 0
        self.richTextBox.AutoSize = True
        self.richTextBox.ScrollBars = WinForms.RichTextBoxScrollBars.ForcedBoth
        self.richTextBox.Font = System.Drawing.Font("Tahoma", 10.0)
        self.richTextBox.AcceptsTab = True
        self.richTextBox.Location = System.Drawing.Point(0, 0)

        self.statusBar.BackColor = System.Drawing.SystemColors.Control
        self.statusBar.Location = System.Drawing.Point(0, 518)
        self.statusBar.Size = System.Drawing.Size(775, 19)
        self.statusBar.TabIndex = 1
        self.statusBar.ShowPanels = True
        self.statusBar.Panels.Add(self.statusBarPanel1)

        items = (self.fileMenu, self.editMenu, self.formatMenu, self.aboutMenu)
        self.mainMenu.MenuItems.AddRange(items)

        self.openFileDialog.Filter = "Text documents|*.txt|RTF document|*.rtf"
        self.openFileDialog.Title = "Open document"

        self.saveFileDialog.Filter = (
            "Text Documents|*.txt|" "Rich Text Format|*.rtf"
        )
        self.saveFileDialog.Title = "Save document"
        self.saveFileDialog.FileName = "Untitled"

        self.AutoScaleBaseSize = System.Drawing.Size(5, 13)
        self.ClientSize = System.Drawing.Size(775, 537)
        self.Menu = self.mainMenu
        self.Text = "Python Wordpad"

        self.Controls.Add(self.statusBar)
        self.Controls.Add(self.richTextBox)
        self.statusBarPanel1.EndInit()

    def Dispose(self):
        self.components.Dispose()
        WinForms.Form.Dispose(self)

    def OnClickFileNew(self, sender, args):
        self.SaveChangesDialog()
        self.NewDocument()

    def OnClickFileOpen(self, sender, args):
        self.SaveChangesDialog()
        self.OpenDocument()

    def OnClickFileSave(self, sender, args):
        self.SaveDocument()

    def OnClickFileSaveAs(self, sender, args):
        self.filename = ""
        self.SaveDocument()

    def OnClickFileExit(self, sender, args):
        self.SaveChangesDialog()
        self.Close()

    def OnClickEditUndo(self, sender, args):
        self.richTextBox.Undo()

    def OnClickEditRedo(self, sender, args):
        self.richTextBox.Redo()

    def OnClickEditCut(self, sender, args):
        self.richTextBox.Cut()

    def OnClickEditCopy(self, sender, args):
        self.richTextBox.Copy()

    def OnClickEditPaste(self, sender, args):
        self.richTextBox.Paste()

    def OnClickEditSelectAll(self, sender, args):
        self.richTextBox.SelectAll()

    def OnClickFormatWordWrap(self, sender, args):
        value = not self.word_wrap
        self.richTextBox.WordWrap = value
        self.menuFormatWordWrap.Checked = value
        self.word_wrap = value

    def OnClickFormatFont(self, sender, args):
        if self.fontDialog.ShowDialog() == WinForms.DialogResult.OK:
            self.richTextBox.SelectionFont = self.fontDialog.Font

    def OnClickHelpAbout(self, sender, args):
        AboutForm().ShowDialog(self)

    def NewDocument(self):
        self.doctype = 1
        self.richTextBox.Rtf = ""
        self.richTextBox.Text = ""
        self.Text = "Python Wordpad - (New Document)"
        self.filename = ""

    def OpenDocument(self):
        if self.openFileDialog.ShowDialog() != WinForms.DialogResult.OK:
            return

        filename = self.openFileDialog.FileName

        stream = File.OpenRead(filename)

        buff = System.Array.CreateInstance(System.Byte, 1024)
        buff.Initialize()
        data = []
        read = 1

        while read > 0:
            read, _ = stream.Read(buff, 0, 1024)
            temp = Encoding.ASCII.GetString(buff, 0, read)
            data.append(temp)

        data = "".join(data)
        stream.Close()

        filename = self.filename = filename.lower()

        if filename.endswith(".rtf"):
            self.richTextBox.Rtf = data
            self.doctype = 2
        else:
            self.richTextBox.Text = data
            self.doctype = 1

        self.Text = "Python Wordpad - %s" % filename
        self.richTextBox.Select(0, 0)

    def SaveDocument(self):
        filename = self.filename

        if not filename:
            if self.saveFileDialog.ShowDialog() != WinForms.DialogResult.OK:
                return
            filename = self.saveFileDialog.FileName

        filename = self.filename = filename.lower()
        self.Text = "Python Wordpad - %s" % filename

        self.richTextBox.Select(0, 0)

        stream = File.OpenWrite(filename)

        if filename.endswith(".rtf"):
            data = self.richTextBox.Rtf
        else:
            data = self.richTextBox.Text

        data = System.Text.Encoding.ASCII.GetBytes(System.String(data))

        stream.Write(data, 0, data.Length)
        stream.Close()

    def SaveChangesDialog(self):
        if self.richTextBox.Modified:
            if (
                WinForms.MessageBox.Show(
                    "Save changes?",
                    "Word Pad",
                    WinForms.MessageBoxButtons.OK
                    | WinForms.MessageBoxButtons.YesNo,
                )
                == WinForms.DialogResult.Yes
            ):
                self.SaveDocument()
                return 1
        return 0


class AboutForm(WinForms.Form):
    def __init__(self):
        super().__init__()
        self.InitializeComponent()

    def InitializeComponent(self):
        """Initialize form components."""
        self.Text = "Python Wordpad"
        self.components = System.ComponentModel.Container()

        self.btnClose = WinForms.Button()
        self.label1 = WinForms.Label()
        self.SuspendLayout()

        self.btnClose.Location = System.Drawing.Point(360, 181)
        self.btnClose.Name = "bnClose"
        self.btnClose.TabIndex = 1
        self.btnClose.Text = "&Close"
        self.btnClose.Click += self.OnClickClose

        self.label1.Location = System.Drawing.Point(20, 20)
        self.label1.Name = "label1"
        self.label1.Size = System.Drawing.Size(296, 140)
        self.label1.TabIndex = 2
        self.label1.Text = (
            "Python Wordpad - an example winforms "
            "application using Python.NET"
        )

        self.AutoScaleBaseSize = System.Drawing.Size(5, 13)
        self.ClientSize = System.Drawing.Size(300, 150)

        self.Controls.AddRange((self.label1, self.btnClose))

        self.FormBorderStyle = WinForms.FormBorderStyle.FixedDialog
        self.MaximizeBox = False
        self.MinimizeBox = False
        self.Name = "AboutForm"
        self.ShowInTaskbar = False
        self.StartPosition = WinForms.FormStartPosition.CenterScreen
        self.Text = "About"
        self.ResumeLayout(False)

    def OnClickClose(self, sender, args):
        self.Close()


def app_thread():
    app = Wordpad()
    WinForms.Application.Run(app)
    app.Dispose()


def main():
    thread = Thread(ThreadStart(app_thread))
    thread.SetApartmentState(ApartmentState.STA)
    thread.Start()
    thread.Join()


if __name__ == "__main__":
    main()



"""
Sample to show the datetime in RFC1123 (timezone is required)

pytz version
"""

from __future__ import annotations

import datetime

import pytz

RFC1123 = "%a, %d %b %Y %H:%M:%S %z"

utc_time = datetime.datetime.now(pytz.utc)
print("UTC time:", utc_time.strftime(RFC1123))

tz1 = pytz.timezone("America/Sao_Paulo")
brz_time = utc_time.astimezone(tz1)
print("Brazil time:", brz_time.strftime(RFC1123))

tz2 = pytz.timezone("US/Eastern")
eas_time = utc_time.astimezone(tz2)
print("US Eastern time:", eas_time.strftime(RFC1123))



from __future__ import annotations

import sqlite3

# examples from python documentation

con = sqlite3.connect(":memory:")
cur = con.cursor()
cur.executescript(
    """
    create table person(
        firstname,
        lastname,
        age
    );

    create table book(
        title,
        author,
        published
    );

    insert into book(title, author, published)
    values (
        'Dirk Gently''s Holistic Detective Agency',
        'Douglas Adams',
        1987
    );
    """
)
with open("dump.sql", "w") as f:
    for line in con.iterdump():
        f.write("%s\n" % line)

print("dump.sql created")
con.close()



"""First script that is run when cx_Freeze starts up. It determines the name of
the initscript that is to be executed after a basic initialization.
"""

from __future__ import annotations

import os
import string
import sys
from importlib.machinery import (
    EXTENSION_SUFFIXES,
    ExtensionFileLoader,
    ModuleSpec,
    PathFinder,
)
from sysconfig import get_platform

import BUILD_CONSTANTS

STRINGREPLACE = list(
    string.whitespace + string.punctuation.replace(".", "").replace("_", "")
)
IS_MINGW = get_platform().startswith("mingw")
IS_WINDOWS = get_platform().startswith("win")


class ExtensionFinder(PathFinder):
    """A Finder for extension modules of packages in zip files."""

    @classmethod
    def find_spec(cls, fullname, path=None, target=None):  # noqa: ARG003
        """Finder only for extension modules found within packages that
        are included in the zip file (instead of as files on disk);
        extension modules cannot be found within zip files but are stored in
        the lib subdirectory; if the extension module is found in a package,
        however, its name has been altered so this finder is needed.
        """
        if path is None:
            return None
        suffixes = EXTENSION_SUFFIXES
        for entry in sys.path:
            if ".zip" in entry:
                continue
            for ext in suffixes:
                location = os.path.join(entry, fullname + ext)
                if os.path.isfile(location):
                    loader = ExtensionFileLoader(fullname, location)
                    return ModuleSpec(fullname, loader, origin=location)
        return None


def init():
    """Basic initialization of the startup script."""
    # update sys module
    sys.executable = os.path.normpath(sys.executable)
    sys.frozen_dir = frozen_dir = os.path.dirname(sys.executable)
    sys.meta_path.append(ExtensionFinder)

    if IS_MINGW:
        sys.path = list(map(os.path.normpath, sys.path))
    if IS_WINDOWS or IS_MINGW:
        # for python >= 3.8, the search for dlls is sandboxed
        search_path: list[str] = [
            entry for entry in sys.path if os.path.isdir(entry)
        ]
        add_to_path = os.path.join(frozen_dir, "lib")
        if add_to_path not in search_path:
            search_path.insert(0, add_to_path)
        # add to dll search path (or to path)
        env_path = os.environ.get("PATH", "").split(os.pathsep)
        if IS_MINGW:
            env_path = list(map(os.path.normpath, env_path))
        for directory in search_path:
            try:
                os.add_dll_directory(directory)
            except OSError:
                pass
            except AttributeError:
                # add to path only when python < 3.8
                if directory not in env_path:
                    env_path.insert(0, directory)
        if IS_MINGW:
            env_path = [entry.replace(os.sep, os.altsep) for entry in env_path]
        os.environ["PATH"] = os.pathsep.join(env_path)

    # set environment variables
    for name in (
        "TCL_LIBRARY",
        "TK_LIBRARY",
        "PYTZ_TZDATADIR",
        "PYTHONTZPATH",
    ):
        try:
            value = getattr(BUILD_CONSTANTS, name)
        except AttributeError:
            pass
        else:
            os.environ[name] = os.path.join(frozen_dir, value)


def run():
    """Determines the name of the initscript and execute it."""
    # get the real name of __init__ script
    # basically, the basename of executable plus __init__
    # but can be renamed when only one executable exists
    name = os.path.normcase(os.path.basename(sys.executable))
    if IS_WINDOWS or IS_MINGW:
        name, _ = os.path.splitext(name)
    name = name.partition(".")[0]
    if not name.isidentifier():
        for char in STRINGREPLACE:
            name = name.replace(char, "_")
    try:
        module_init = __import__(name + "__init__")
    except ModuleNotFoundError:
        names = [
            f.rpartition("__init__")[0]
            for f in __loader__._files  # pylint: disable=protected-access
            if f.endswith("__init__.pyc")
            and f.rpartition("__init__")[0].isidentifier()
        ]
        if len(names) != 1:
            raise RuntimeError(
                "Apparently, the original executable has been renamed to "
                f"{name!r}. When multiple executables are generated, "
                "renaming is not allowed."
            ) from None
        name = names[0]
        module_init = __import__(name + "__init__")
    module_init.run(name + "__main__")



"""Implements the bdist_msi command.

Borrowed from distutils.command.bdist_msi of Python 3.8
"""
# Copyright (C) 2005, 2006 Martin von Löwis
# Licensed to PSF under a Contributor Agreement.

from __future__ import annotations

import logging
import os
import shutil
import sys
from msilib import (  # pylint: disable=deprecated-module
    CAB,
    Binary,
    Dialog,
    Directory,
    Feature,
    Win64,
    add_data,
    add_tables,
    gen_uuid,
    init_database,
    schema,
    sequence,
    text,
)
from sysconfig import get_platform, get_python_version

from setuptools import Command

from .._compat import packaging
from ..exception import OptionError
from ._pydialog import PyDialog


# pylint: disable=attribute-defined-outside-init,invalid-name
class bdist_msi(Command):
    """Create a Microsoft Installer (.msi) binary distribution."""

    description = __doc__

    user_options = [
        (
            "bdist-dir=",
            None,
            "temporary directory for creating the distribution",
        ),
        (
            "plat-name=",
            "p",
            "platform name to embed in generated filenames "
            f"(default: {get_platform()})",
        ),
        (
            "keep-temp",
            "k",
            "keep the pseudo-installation tree around after "
            "creating the distribution archive",
        ),
        (
            "target-version=",
            None,
            "require a specific python version on the target system",
        ),
        (
            "no-target-compile",
            "c",
            "do not compile .py to .pyc on the target system",
        ),
        (
            "no-target-optimize",
            "o",
            "do not compile .py to .pyo (optimized) on the target system",
        ),
        ("dist-dir=", "d", "directory to put final built distributions in"),
        (
            "skip-build",
            None,
            "skip rebuilding everything (for testing/debugging)",
        ),
        (
            "install-script=",
            None,
            "basename of installation script to be run after "
            "installation or before deinstallation",
        ),
        (
            "pre-install-script=",
            None,
            "Fully qualified filename of a script to be run before "
            "any files are installed.  This script need not be in the "
            "distribution",
        ),
    ]

    boolean_options = [
        "keep-temp",
        "no-target-compile",
        "no-target-optimize",
        "skip-build",
    ]

    all_versions = [str(i / 10) for i in range(20, 39)]  # 2.0 to 3.9
    all_versions += ["3.10", "3.11", "3.12", "3.13"]
    other_version = "X"

    def initialize_options(self):
        self.bdist_dir = None
        self.plat_name = None
        self.keep_temp = 0
        self.no_target_compile = 0
        self.no_target_optimize = 0
        self.target_version = None
        self.dist_dir = None
        self.skip_build = None
        self.install_script = None
        self.pre_install_script = None
        self.versions = None

    def finalize_options(self):
        self.set_undefined_options("bdist", ("skip_build", "skip_build"))

        if self.bdist_dir is None:
            bdist_base = self.get_finalized_command("bdist").bdist_base
            self.bdist_dir = os.path.join(bdist_base, "msi")

        short_version = get_python_version()
        if (not self.target_version) and self.distribution.has_ext_modules():
            self.target_version = short_version

        if self.target_version:
            self.versions = [self.target_version]
            if (
                not self.skip_build
                and self.distribution.has_ext_modules()
                and self.target_version != short_version
            ):
                raise OptionError(
                    f"target version can only be {short_version}, or the "
                    "'--skip-build' option must be specified"
                )
        else:
            self.versions = list(self.all_versions)

        self.set_undefined_options(
            "bdist",
            ("dist_dir", "dist_dir"),
            ("plat_name", "plat_name"),
        )

        if self.pre_install_script:
            raise OptionError(
                "the pre-install-script feature is not yet implemented"
            )

        if self.install_script:
            for script in self.distribution.scripts:
                if self.install_script == os.path.basename(script):
                    break
            else:
                raise OptionError(
                    f"install_script '{self.install_script}' not found in "
                    "scripts"
                )
        self.install_script_key = None

    def run(self):
        if not self.skip_build:
            self.run_command("build")

        install = self.reinitialize_command("install", reinit_subcommands=1)
        install.prefix = self.bdist_dir
        install.skip_build = self.skip_build
        install.warn_dir = 0

        install_lib = self.reinitialize_command("install_lib")
        # we do not want to include pyc or pyo files
        install_lib.compile = 0
        install_lib.optimize = 0

        if self.distribution.has_ext_modules():
            # If we are building an installer for a Python version other
            # than the one we are currently running, then we need to ensure
            # our build_lib reflects the other Python version rather than ours.
            # Note that for target_version!=sys.version, we must have skipped
            # the build step, so there is no issue with enforcing the build of
            # this version.
            target_version = self.target_version
            if not target_version:
                assert self.skip_build, "Should have already checked this"
                target_version = get_python_version()
            plat_specifier = f".{self.plat_name}-{target_version}"
            build = self.get_finalized_command("build")
            build.build_lib = os.path.join(
                build.build_base, "lib" + plat_specifier
            )

        logging.info("installing to %s", self.bdist_dir)
        install.ensure_finalized()

        # avoid warning of 'install_lib' about installing
        # into a directory not in sys.path
        sys.path.insert(0, os.path.join(self.bdist_dir, "PURELIB"))

        install.run()

        del sys.path[0]

        self.mkpath(self.dist_dir)
        fullname = self.distribution.get_fullname()
        installer_name = self.get_installer_filename(fullname)
        installer_name = os.path.abspath(installer_name)
        if os.path.exists(installer_name):
            os.unlink(installer_name)

        metadata = self.distribution.metadata
        author = metadata.get_contact() or "UNKNOWN"
        version = metadata.get_version()
        # ProductVersion must be strictly numeric
        # XXX need to deal with prerelease versions
        base_version = packaging.version.Version(version).base_version
        # Prefix ProductName with Python x.y, so that
        # it sorts together with the other Python packages
        # in Add-Remove-Programs (APR)
        fullname = self.distribution.get_fullname()
        if self.target_version:
            product_name = f"Python {self.target_version} {fullname}"
        else:
            product_name = f"Python {fullname}"
        self.db = init_database(
            installer_name,
            schema,
            product_name,
            gen_uuid(),
            base_version,
            author,
        )
        add_tables(self.db, sequence)
        props = [("DistVersion", version)]
        email = metadata.author_email or metadata.maintainer_email
        if email:
            props.append(("ARPCONTACT", email))
        if metadata.url:
            props.append(("ARPURLINFOABOUT", metadata.url))
        if props:
            add_data(self.db, "Property", props)

        self.add_find_python()
        self.add_files()
        self.add_scripts()
        self.add_ui()
        self.db.Commit()

        if hasattr(self.distribution, "dist_files"):
            tup = "bdist_msi", self.target_version or "any", fullname
            self.distribution.dist_files.append(tup)

        if not self.keep_temp:
            bdist_dir = self.bdist_dir
            logging.info("removing '%s' (and everything under it)", bdist_dir)
            if not self.dry_run:
                try:
                    shutil.rmtree(bdist_dir)
                except OSError as exc:
                    logging.warning("error removing %s: %s", bdist_dir, exc)

    def add_files(self):
        db = self.db
        cab = CAB("distfiles")
        rootdir = os.path.abspath(self.bdist_dir)

        root = Directory(db, cab, None, rootdir, "TARGETDIR", "SourceDir")
        feat = Feature(
            db, "Python", "Python", "Everything", 0, 1, directory="TARGETDIR"
        )

        items = [(feat, root, "")]
        for version in [*self.versions, self.other_version]:
            target = "TARGETDIR" + version
            name = default = "Python" + version
            desc = "Everything"
            if version is self.other_version:
                title = "Python from another location"
                level = 2
            else:
                title = f"Python {version} from registry"
                level = 1
            feat = Feature(db, name, title, desc, 1, level, directory=target)
            directory = Directory(db, cab, root, rootdir, target, default)
            items.append((feat, directory, version))
        db.Commit()

        seen = {}
        for feature, base_directory, version in items:
            todo = [base_directory]
            while todo:
                directory = todo.pop()
                for file in os.listdir(directory.absolute):
                    afile = os.path.join(directory.absolute, file)
                    if os.path.isdir(afile):
                        short = f"{directory.make_short(file)}|{file}"
                        default = file + version
                        newdir = Directory(
                            db, cab, directory, file, default, short
                        )
                        todo.append(newdir)
                    else:
                        if not directory.component:
                            directory.start_component(
                                directory.logical, feature, 0
                            )
                        if afile not in seen:
                            key = seen[afile] = directory.add_file(file)
                            if file == self.install_script:
                                if self.install_script_key:
                                    raise OptionError(
                                        f"Multiple files with name {file}"
                                    )
                                self.install_script_key = f"[#{key}]"
                        else:
                            key = seen[afile]
                            add_data(
                                self.db,
                                "DuplicateFile",
                                [
                                    (
                                        key + version,
                                        directory.component,
                                        key,
                                        None,
                                        directory.logical,
                                    )
                                ],
                            )
            db.Commit()
        cab.commit(db)

    def add_find_python(self):
        r"""Adds code to the installer to compute the location of Python.

        Properties PYTHON.MACHINE.X.Y and PYTHON.USER.X.Y will be set from the
        registry for each version of Python.

        Properties TARGETDIRX.Y will be set from PYTHON.USER.X.Y if defined,
        else from PYTHON.MACHINE.X.Y.

        Properties PYTHONX.Y will be set to TARGETDIRX.Y\python.exe
        """
        start = 402
        for ver in self.versions:
            install_path = rf"SOFTWARE\Python\PythonCore\{ver}\InstallPath"
            machine_reg = f"python.machine.{ver}"
            user_reg = f"python.user.{ver}"
            machine_prop = f"PYTHON.MACHINE.{ver}"
            user_prop = f"PYTHON.USER.{ver}"
            machine_action = f"PythonFromMachine{ver}"
            user_action = f"PythonFromUser{ver}"
            exe_action = f"PythonExe{ver}"
            target_dir_prop = f"TARGETDIR{ver}"
            exe_prop = f"PYTHON{ver}"
            # type_: msidbLocatorTypeRawValue [ + msidbLocatorType64bit ]
            type_ = 2 + 16 if Win64 else 2
            add_data(
                self.db,
                "RegLocator",
                [
                    (machine_reg, 2, install_path, None, type_),
                    (user_reg, 1, install_path, None, type_),
                ],
            )
            add_data(
                self.db,
                "AppSearch",
                [(machine_prop, machine_reg), (user_prop, user_reg)],
            )
            add_data(
                self.db,
                "CustomAction",
                [
                    (
                        machine_action,
                        51 + 256,
                        target_dir_prop,
                        f"[{machine_prop}]",
                    ),
                    (
                        user_action,
                        51 + 256,
                        target_dir_prop,
                        f"[{user_prop}]",
                    ),
                    (
                        exe_action,
                        51 + 256,
                        exe_prop,
                        f"[{target_dir_prop}]\\python.exe",
                    ),
                ],
            )
            add_data(
                self.db,
                "InstallExecuteSequence",
                [
                    (machine_action, machine_prop, start),
                    (user_action, user_prop, start + 1),
                    (exe_action, None, start + 2),
                ],
            )
            add_data(
                self.db,
                "InstallUISequence",
                [
                    (machine_action, machine_prop, start),
                    (user_action, user_prop, start + 1),
                    (exe_action, None, start + 2),
                ],
            )
            add_data(
                self.db,
                "Condition",
                [(f"Python{ver}", 0, f"NOT TARGETDIR{ver}")],
            )
            start += 4
            assert start < 500

    def add_scripts(self):
        if self.install_script:
            start = 6800
            for ver in [*self.versions, self.other_version]:
                install_action = f"install_script.{ver}"
                exe_prop = f"PYTHON{ver}"
                add_data(
                    self.db,
                    "CustomAction",
                    [(install_action, 50, exe_prop, self.install_script_key)],
                )
                add_data(
                    self.db,
                    "InstallExecuteSequence",
                    [(install_action, f"&Python{ver}=3", start)],
                )
                start += 1
        # XXX pre-install scripts are currently refused in finalize_options()
        #     but if this feature is completed, it will also need to add
        #     entries for each version as the above code does
        if self.pre_install_script:
            scriptfn = os.path.join(self.bdist_dir, "preinstall.bat")
            with open(scriptfn, "w", encoding="utf-8") as file:
                # The batch file will be executed with [PYTHON], so that %1
                # is the path to the Python interpreter; %0 will be the path
                # of the batch file.
                # rem ="""
                # %1 %0
                # exit
                # """
                # <actual script>
                file.write('rem ="""\n%1 %0\nexit\n"""\n')
                with open(self.pre_install_script, encoding="utf-8") as fin:
                    file.write(fin.read())
            add_data(self.db, "Binary", [("PreInstall", Binary(scriptfn))])
            add_data(
                self.db,
                "CustomAction",
                [("PreInstall", 2, "PreInstall", None)],
            )
            add_data(
                self.db,
                "InstallExecuteSequence",
                [("PreInstall", "NOT Installed", 450)],
            )

    def add_ui(self):
        db = self.db
        x = y = 50
        w = 370
        h = 300
        title = "[ProductName] Setup"

        # see "Dialog Style Bits"
        modal = 3  # visible | modal
        modeless = 1  # visible
        # track_disk_space = 32

        # UI customization properties
        add_data(
            db,
            "Property",
            # See "DefaultUIFont Property"
            [
                ("DefaultUIFont", "DlgFont8"),
                # See "ErrorDialog Style Bit"
                ("ErrorDialog", "ErrorDlg"),
                ("Progress1", "Install"),  # modified in maintenance type dlg
                ("Progress2", "installs"),
                ("MaintenanceForm_Action", "Repair"),
                # possible values: ALL, JUSTME
                ("WhichUsers", "ALL"),
            ],
        )

        # Fonts, see "TextStyle Table"
        add_data(
            db,
            "TextStyle",
            [
                ("DlgFont8", "Tahoma", 9, None, 0),
                ("DlgFontBold8", "Tahoma", 8, None, 1),  # bold
                ("VerdanaBold10", "Verdana", 10, None, 1),
                ("VerdanaRed9", "Verdana", 9, 255, 0),
            ],
        )

        # UI Sequences, see "InstallUISequence Table", "Using a Sequence Table"
        # Numbers indicate sequence; see sequence.py for how these action
        # integrate
        add_data(
            db,
            "InstallUISequence",
            [
                (
                    "PrepareDlg",
                    "Not Privileged or Windows9x or Installed",
                    140,
                ),
                (
                    "WhichUsersDlg",
                    "Privileged and not Windows9x and not Installed",
                    141,
                ),
                # In the user interface, assume all-users installation if
                # privileged.
                ("SelectFeaturesDlg", "Not Installed", 1230),
                # XXX no support for resume installations yet
                # ("ResumeDlg", "Installed AND (RESUME OR Preselected)", 1240),
                (
                    "MaintenanceTypeDlg",
                    "Installed AND NOT RESUME AND NOT Preselected",
                    1250,
                ),
                ("ProgressDlg", None, 1280),
            ],
        )

        add_data(db, "ActionText", text.ActionText)
        add_data(db, "UIText", text.UIText)
        #####################################################################
        # Standard dialogs: FatalError, UserExit, ExitDialog
        fatal = PyDialog(
            db,
            "FatalError",
            x,
            y,
            w,
            h,
            modal,
            title,
            "Finish",
            "Finish",
            "Finish",
        )
        fatal.title("[ProductName] Installer ended prematurely")
        fatal.backbutton("< Back", "Finish", active=0)
        fatal.cancelbutton("Cancel", "Back", active=0)
        fatal.text(
            "Description1",
            15,
            70,
            320,
            80,
            0x30003,
            "[ProductName] setup ended prematurely because of an error."
            "  Your system has not been modified.  To install this program "
            "at a later time, please run the installation again.",
        )
        fatal.text(
            "Description2",
            15,
            155,
            320,
            20,
            0x30003,
            "Click the Finish button to exit the Installer.",
        )
        c = fatal.nextbutton("Finish", "Cancel", name="Finish")
        c.event("EndDialog", "Exit")

        user_exit = PyDialog(
            db,
            "UserExit",
            x,
            y,
            w,
            h,
            modal,
            title,
            "Finish",
            "Finish",
            "Finish",
        )
        user_exit.title("[ProductName] Installer was interrupted")
        user_exit.backbutton("< Back", "Finish", active=0)
        user_exit.cancelbutton("Cancel", "Back", active=0)
        user_exit.text(
            "Description1",
            15,
            70,
            320,
            80,
            0x30003,
            "[ProductName] setup was interrupted.  Your system has not been "
            "modified.  To install this program at a later time, please run "
            "the installation again.",
        )
        user_exit.text(
            "Description2",
            15,
            155,
            320,
            20,
            0x30003,
            "Click the Finish button to exit the Installer.",
        )
        c = user_exit.nextbutton("Finish", "Cancel", name="Finish")
        c.event("EndDialog", "Exit")

        exit_dialog = PyDialog(
            db,
            "ExitDialog",
            x,
            y,
            w,
            h,
            modal,
            title,
            "Finish",
            "Finish",
            "Finish",
        )
        exit_dialog.title("Completing the [ProductName] Installer")
        exit_dialog.backbutton("< Back", "Finish", active=0)
        exit_dialog.cancelbutton("Cancel", "Back", active=0)
        exit_dialog.text(
            "Description",
            15,
            235,
            320,
            20,
            0x30003,
            "Click the Finish button to exit the Installer.",
        )
        c = exit_dialog.nextbutton("Finish", "Cancel", name="Finish")
        c.event("EndDialog", "Return")

        #####################################################################
        # Required dialog: FilesInUse, ErrorDlg
        inuse = PyDialog(
            db,
            "FilesInUse",
            x,
            y,
            w,
            h,
            19,  # KeepModeless|Modal|Visible
            title,
            "Retry",
            "Retry",
            "Retry",
            bitmap=False,
        )
        inuse.text(
            "Title", 15, 6, 200, 15, 0x30003, r"{\DlgFontBold8}Files in Use"
        )
        inuse.text(
            "Description",
            20,
            23,
            280,
            20,
            0x30003,
            "Some files that need to be updated are currently in use.",
        )
        inuse.text(
            "Text",
            20,
            55,
            330,
            50,
            3,
            "The following applications are using files that need to be "
            "updated by this setup. Close these applications and then click "
            "Retry to continue the installation or Cancel to exit it.",
        )
        inuse.control(
            "List",
            "ListBox",
            20,
            107,
            330,
            130,
            7,
            "FileInUseProcess",
            None,
            None,
            None,
        )
        c = inuse.backbutton("Exit", "Ignore", name="Exit")
        c.event("EndDialog", "Exit")
        c = inuse.nextbutton("Ignore", "Retry", name="Ignore")
        c.event("EndDialog", "Ignore")
        c = inuse.cancelbutton("Retry", "Exit", name="Retry")
        c.event("EndDialog", "Retry")

        # See "Error Dialog". See "ICE20" for the required names of the
        # controls.
        error = Dialog(
            db,
            "ErrorDlg",
            50,
            10,
            330,
            101,
            65543,  # Error|Minimize|Modal|Visible
            title,
            "ErrorText",
            None,
            None,
        )
        error.text("ErrorText", 50, 9, 280, 48, 3, "")
        # error.control("ErrorIcon", "Icon", 15, 9, 24, 24, 5242881, None,
        # "py.ico", None, None)
        error.pushbutton("N", 120, 72, 81, 21, 3, "No", None).event(
            "EndDialog", "ErrorNo"
        )
        error.pushbutton("Y", 240, 72, 81, 21, 3, "Yes", None).event(
            "EndDialog", "ErrorYes"
        )
        error.pushbutton("A", 0, 72, 81, 21, 3, "Abort", None).event(
            "EndDialog", "ErrorAbort"
        )
        error.pushbutton("C", 42, 72, 81, 21, 3, "Cancel", None).event(
            "EndDialog", "ErrorCancel"
        )
        error.pushbutton("I", 81, 72, 81, 21, 3, "Ignore", None).event(
            "EndDialog", "ErrorIgnore"
        )
        error.pushbutton("O", 159, 72, 81, 21, 3, "Ok", None).event(
            "EndDialog", "ErrorOk"
        )
        error.pushbutton("R", 198, 72, 81, 21, 3, "Retry", None).event(
            "EndDialog", "ErrorRetry"
        )

        #####################################################################
        # Global "Query Cancel" dialog
        cancel = Dialog(
            db, "CancelDlg", 50, 10, 260, 85, 3, title, "No", "No", "No"
        )
        cancel.text(
            "Text",
            48,
            15,
            194,
            30,
            3,
            "Are you sure you want to cancel [ProductName] installation?",
        )
        # cancel.control("Icon", "Icon", 15, 15, 24, 24, 5242881, None,
        #               "py.ico", None, None)
        c = cancel.pushbutton("Yes", 72, 57, 56, 17, 3, "Yes", "No")
        c.event("EndDialog", "Exit")

        c = cancel.pushbutton("No", 132, 57, 56, 17, 3, "No", "Yes")
        c.event("EndDialog", "Return")

        #####################################################################
        # Global "Wait for costing" dialog
        costing = Dialog(
            db,
            "WaitForCostingDlg",
            50,
            10,
            260,
            85,
            modal,
            title,
            "Return",
            "Return",
            "Return",
        )
        costing.text(
            "Text",
            48,
            15,
            194,
            30,
            3,
            "Please wait while the installer finishes determining your disk "
            "space requirements.",
        )
        c = costing.pushbutton("Return", 102, 57, 56, 17, 3, "Return", None)
        c.event("EndDialog", "Exit")

        #####################################################################
        # Preparation dialog: no user input except cancellation
        prep = PyDialog(
            db,
            "PrepareDlg",
            x,
            y,
            w,
            h,
            modeless,
            title,
            "Cancel",
            "Cancel",
            "Cancel",
        )
        prep.text(
            "Description",
            15,
            70,
            320,
            40,
            0x30003,
            "Please wait while the Installer prepares to guide you through "
            "the installation.",
        )
        prep.title("Welcome to the [ProductName] Installer")
        c = prep.text("ActionText", 15, 110, 320, 20, 0x30003, "Pondering...")
        c.mapping("ActionText", "Text")
        c = prep.text("ActionData", 15, 135, 320, 30, 0x30003, None)
        c.mapping("ActionData", "Text")
        prep.backbutton("Back", None, active=0)
        prep.nextbutton("Next", None, active=0)
        c = prep.cancelbutton("Cancel", None)
        c.event("SpawnDialog", "CancelDlg")

        #####################################################################
        # Feature (Python directory) selection
        seldlg = PyDialog(
            db,
            "SelectFeaturesDlg",
            x,
            y,
            w,
            h,
            modal,
            title,
            "Next",
            "Next",
            "Cancel",
        )
        seldlg.title("Select Python Installations")

        seldlg.text(
            "Hint",
            15,
            30,
            300,
            20,
            3,
            "Select the Python locations where "
            f"{self.distribution.get_fullname()} should be installed.",
        )

        seldlg.backbutton("< Back", None, active=0)
        c = seldlg.nextbutton("Next >", "Cancel")
        order = 1
        c.event("[TARGETDIR]", "[SourceDir]", ordering=order)
        for version in [*self.versions, self.other_version]:
            order += 1
            c.event(
                "[TARGETDIR]",
                f"[TARGETDIR{version}]",
                f"FEATURE_SELECTED AND &Python{version}=3",
                ordering=order,
            )
        c.event("SpawnWaitDialog", "WaitForCostingDlg", ordering=order + 1)
        c.event("EndDialog", "Return", ordering=order + 2)
        c = seldlg.cancelbutton("Cancel", "Features")
        c.event("SpawnDialog", "CancelDlg")

        c = seldlg.control(
            "Features",
            "SelectionTree",
            15,
            60,
            300,
            120,
            3,
            "FEATURE",
            None,
            "PathEdit",
            None,
        )
        c.event("[FEATURE_SELECTED]", "1")
        ver = self.other_version
        install_other_cond = f"FEATURE_SELECTED AND &Python{ver}=3"
        dont_install_other_cond = f"FEATURE_SELECTED AND &Python{ver}<>3"

        c = seldlg.text(
            "Other",
            15,
            200,
            300,
            15,
            3,
            "Provide an alternate Python location",
        )
        c.condition("Enable", install_other_cond)
        c.condition("Show", install_other_cond)
        c.condition("Disable", dont_install_other_cond)
        c.condition("Hide", dont_install_other_cond)

        c = seldlg.control(
            "PathEdit",
            "PathEdit",
            15,
            215,
            300,
            16,
            1,
            "TARGETDIR" + ver,
            None,
            "Next",
            None,
        )
        c.condition("Enable", install_other_cond)
        c.condition("Show", install_other_cond)
        c.condition("Disable", dont_install_other_cond)
        c.condition("Hide", dont_install_other_cond)

        #####################################################################
        # Disk cost
        cost = PyDialog(
            db,
            "DiskCostDlg",
            x,
            y,
            w,
            h,
            modal,
            title,
            "OK",
            "OK",
            "OK",
            bitmap=False,
        )
        cost.text(
            "Title",
            15,
            6,
            200,
            15,
            0x30003,
            r"{\DlgFontBold8}Disk Space Requirements",
        )
        cost.text(
            "Description",
            20,
            20,
            280,
            20,
            0x30003,
            "The disk space required for the installation of the selected "
            "features.",
        )
        cost.text(
            "Text",
            20,
            53,
            330,
            60,
            3,
            "The highlighted volumes (if any) do not have enough disk space "
            "available for the currently selected features.  You can either "
            "remove some files from the highlighted volumes, or choose to "
            "install less features onto local drive(s), or select different "
            "destination drive(s).",
        )
        cost.control(
            "VolumeList",
            "VolumeCostList",
            20,
            100,
            330,
            150,
            393223,
            None,
            "{120}{70}{70}{70}{70}",
            None,
            None,
        )
        cost.xbutton("OK", "Ok", None, 0.5).event("EndDialog", "Return")

        #####################################################################
        # WhichUsers Dialog. Only available on NT, and for privileged users.
        # This must be run before FindRelatedProducts, because that will
        # take into account whether the previous installation was per-user
        # or per-machine. We currently don't support going back to this
        # dialog after "Next" was selected; to support this, we would need to
        # find how to reset the ALLUSERS property, and how to re-run
        # FindRelatedProducts.
        # On Windows9x, the ALLUSERS property is ignored on the command line
        # and in the Property table, but installer fails according to the
        # documentation if a dialog attempts to set ALLUSERS.
        whichusers = PyDialog(
            db,
            "WhichUsersDlg",
            x,
            y,
            w,
            h,
            modal,
            title,
            "AdminInstall",
            "Next",
            "Cancel",
        )
        whichusers.title(
            "Select whether to install [ProductName] for all users of this "
            "computer."
        )
        # A radio group with two options: allusers, justme
        g = whichusers.radiogroup(
            "AdminInstall", 15, 60, 260, 50, 3, "WhichUsers", "", "Next"
        )
        g.add("ALL", 0, 5, 150, 20, "Install for all users")
        g.add("JUSTME", 0, 25, 150, 20, "Install just for me")

        whichusers.backbutton("Back", None, active=0)

        c = whichusers.nextbutton("Next >", "Cancel")
        c.event("[ALLUSERS]", "1", 'WhichUsers="ALL"', 1)
        c.event("EndDialog", "Return", ordering=2)

        c = whichusers.cancelbutton("Cancel", "AdminInstall")
        c.event("SpawnDialog", "CancelDlg")

        #####################################################################
        # Installation Progress dialog (modeless)
        progress = PyDialog(
            db,
            "ProgressDlg",
            x,
            y,
            w,
            h,
            modeless,
            title,
            "Cancel",
            "Cancel",
            "Cancel",
            bitmap=False,
        )
        progress.text(
            "Title",
            20,
            15,
            200,
            15,
            0x30003,
            r"{\DlgFontBold8}[Progress1] [ProductName]",
        )
        progress.text(
            "Text",
            35,
            65,
            300,
            30,
            3,
            "Please wait while the Installer [Progress2] [ProductName]. "
            "This may take several minutes.",
        )
        progress.text("StatusLabel", 35, 100, 35, 20, 3, "Status:")

        c = progress.text("ActionText", 70, 100, w - 70, 20, 3, "Pondering...")
        c.mapping("ActionText", "Text")

        # c=progress.text("ActionData", 35, 140, 300, 20, 3, None)
        # c.mapping("ActionData", "Text")

        c = progress.control(
            "ProgressBar",
            "ProgressBar",
            35,
            120,
            300,
            10,
            65537,
            None,
            "Progress done",
            None,
            None,
        )
        c.mapping("SetProgress", "Progress")

        progress.backbutton("< Back", "Next", active=False)
        progress.nextbutton("Next >", "Cancel", active=False)
        progress.cancelbutton("Cancel", "Back").event(
            "SpawnDialog", "CancelDlg"
        )

        ###################################################################
        # Maintenance type: repair/uninstall
        maint = PyDialog(
            db,
            "MaintenanceTypeDlg",
            x,
            y,
            w,
            h,
            modal,
            title,
            "Next",
            "Next",
            "Cancel",
        )
        maint.title("Welcome to the [ProductName] Setup Wizard")
        maint.text(
            "BodyText",
            15,
            63,
            330,
            42,
            3,
            "Select whether you want to repair or remove [ProductName].",
        )
        g = maint.radiogroup(
            "RepairRadioGroup",
            15,
            108,
            330,
            60,
            3,
            "MaintenanceForm_Action",
            "",
            "Next",
        )
        # g.add("Change", 0, 0, 200, 17, "&Change [ProductName]")
        g.add("Repair", 0, 18, 200, 17, "&Repair [ProductName]")
        g.add("Remove", 0, 36, 200, 17, "Re&move [ProductName]")

        maint.backbutton("< Back", None, active=False)
        c = maint.nextbutton("Finish", "Cancel")
        # Change installation: Change progress dialog to "Change", then ask
        # for feature selection
        # c.event("[Progress1]", "Change", 'MaintenanceForm_Action="Change"',1)
        # c.event("[Progress2]", "changes",'MaintenanceForm_Action="Change"',2)

        # Reinstall: Change progress dialog to "Repair", then invoke reinstall
        # Also set list of reinstalled features to "ALL"
        c.event("[REINSTALL]", "ALL", 'MaintenanceForm_Action="Repair"', 5)
        c.event(
            "[Progress1]", "Repairing", 'MaintenanceForm_Action="Repair"', 6
        )
        c.event("[Progress2]", "repairs", 'MaintenanceForm_Action="Repair"', 7)
        c.event("Reinstall", "ALL", 'MaintenanceForm_Action="Repair"', 8)

        # Uninstall: Change progress to "Remove", then invoke uninstall
        # Also set list of removed features to "ALL"
        c.event("[REMOVE]", "ALL", 'MaintenanceForm_Action="Remove"', 11)
        c.event(
            "[Progress1]", "Removing", 'MaintenanceForm_Action="Remove"', 12
        )
        c.event(
            "[Progress2]", "removes", 'MaintenanceForm_Action="Remove"', 13
        )
        c.event("Remove", "ALL", 'MaintenanceForm_Action="Remove"', 14)

        # Close dialog when maintenance action scheduled
        c.event("EndDialog", "Return", 'MaintenanceForm_Action<>"Change"', 20)
        # c.event("NewDialog", "SelectFeaturesDlg",
        # 'MaintenanceForm_Action="Change"', 21)

        maint.cancelbutton("Cancel", "RepairRadioGroup").event(
            "SpawnDialog", "CancelDlg"
        )

    def get_installer_filename(self, fullname):
        # Factored out to allow overriding in subclasses
        if self.target_version:
            base_name = (
                f"{fullname}.{self.plat_name}-py{self.target_version}.msi"
            )
        else:
            base_name = f"{fullname}.{self.plat_name}.msi"
        return os.path.join(self.dist_dir, base_name)



"""A collection of functions which are triggered automatically by finder when
pythonnet package is included.
"""

from __future__ import annotations

from ..finder import ModuleFinder
from ..module import Module


def load_clr(finder: ModuleFinder, module: Module) -> None:
    """The pythonnet package (imported as 'clr') needs Python.Runtime.dll
    in runtime.
    """
    dll_name = "Python.Runtime.dll"
    dll_path = module.file.parent / dll_name
    if not dll_path.exists():
        dll_path = module.file.parent / "pythonnet/runtime" / dll_name
        if not dll_path.exists():
            return
    finder.include_files(dll_path, f"lib/{dll_name}")



"""A very simple setup script to create a single executable.

hello.py is a very simple 'Hello, world' type script which also displays the
environment in which the script runs.

Run the build process by running the command 'python setup.py build'

If everything works well you should find a subdirectory in the build
subdirectory that contains the files needed to run the script without Python"""

from __future__ import annotations

from cx_Freeze import Executable, setup

executables = [Executable("hello.py")]

setup(
    name="hello",
    version="0.1.2.3",
    description="Sample cx_Freeze script",
    executables=executables,
)



# A simple setup script to create an executable running wxPython. This also
# demonstrates the method for creating a Windows executable that does not have
# an associated console.
#
# wxapp.py is a very simple 'Hello, world' type wxPython application
#
# Run the build process by running the command 'python setup.py build'
#
# If everything works well you should find a subdirectory in the build
# subdirectory that contains the files needed to run the application

from __future__ import annotations

import sys

from cx_Freeze import Executable, setup

base = None
if sys.platform == "win32":
    base = "Win32GUI"

executables = [Executable("wxapp.py", base=base)]

setup(
    name="hello",
    version="0.1",
    description="Sample cx_Freeze wxPython script",
    executables=executables,
)



"""A collection of functions which are triggered automatically by finder when
PySide2 package is included.
"""

from __future__ import annotations

import os

from ..._compat import IS_CONDA, IS_LINUX, IS_WINDOWS
from ...common import get_resource_file_path
from ...finder import ModuleFinder
from ...module import Module
from .._qthooks import copy_qt_files
from .._qthooks import load_qt_qt as load_pyside2_qt
from .._qthooks import load_qt_qtcharts as load_pyside2_qtcharts
from .._qthooks import (
    load_qt_qtdatavisualization as load_pyside2_qtdatavisualization,
)
from .._qthooks import load_qt_qtdesigner as load_pyside2_qtdesigner
from .._qthooks import load_qt_qtgui as load_pyside2_qtgui
from .._qthooks import load_qt_qthelp as load_pyside2_qthelp
from .._qthooks import load_qt_qtlocation as load_pyside2_qtlocation
from .._qthooks import load_qt_qtmultimedia as load_pyside2_qtmultimedia
from .._qthooks import (
    load_qt_qtmultimediawidgets as load_pyside2_qtmultimediawidgets,
)
from .._qthooks import load_qt_qtopengl as load_pyside2_qtopengl
from .._qthooks import load_qt_qtpositioning as load_pyside2_qtpositioning
from .._qthooks import load_qt_qtprintsupport as load_pyside2_qtprintsupport
from .._qthooks import load_qt_qtqml as load_pyside2_qtqml
from .._qthooks import load_qt_qtscripttools as load_pyside2_qtscripttools
from .._qthooks import load_qt_qtsql as load_pyside2_qtsql
from .._qthooks import load_qt_qtsvg as load_pyside2_qtsvg
from .._qthooks import load_qt_qttest as load_pyside2_qttest
from .._qthooks import load_qt_qtuitools as load_pyside2_qtuitools
from .._qthooks import load_qt_qtwebengine as load_pyside2_qtwebengine
from .._qthooks import load_qt_qtwebenginecore as load_pyside2_qtwebenginecore
from .._qthooks import (
    load_qt_qtwebenginewidgets as load_pyside2_qtwebenginewidgets,
)
from .._qthooks import load_qt_qtwebkit as load_pyside2_qtwebkit
from .._qthooks import load_qt_qtwebsockets as load_pyside2_qtwebsockets
from .._qthooks import load_qt_qtwidgets as load_pyside2_qtwidgets
from .._qthooks import load_qt_qtxmlpatterns as load_pyside2_qtxmlpatterns
from .._qthooks import load_qt_uic as load_pyside2_uic


def load_pyside2(finder: ModuleFinder, module: Module) -> None:
    """Inject code in PySide2 __init__ to locate and load plugins and
    resources. Also, this fixes issues with conda-forge versions.
    """
    # Include QtCore module needed by all modules
    finder.include_module("PySide2.QtCore")

    # Activate an optimized mode when PySide2 is in zip_include_packages
    if module.in_file_system == 0:
        module.in_file_system = 2

    # Include modules that inject an optional debug code
    qt_debug = get_resource_file_path("hooks/pyside2", "debug", ".py")
    finder.include_file_as_module(qt_debug, "PySide2._cx_freeze_qt_debug")

    # Include a resource with qt.conf for conda-forge windows/linux
    if IS_CONDA:
        resource = get_resource_file_path("hooks/pyside2", "resource", ".py")
        finder.include_file_as_module(resource, "PySide2._cx_freeze_resource")

    # Include a copy of qt.conf (used by webengine)
    if IS_WINDOWS or IS_LINUX:
        copy_qt_files(finder, "PySide2", "LibraryExecutablesPath", "qt.conf")

    # Inject code to init
    code_string = module.file.read_text(encoding="utf-8")
    code_string += "\n# cx_Freeze patch start\n"
    if IS_CONDA:
        code_string += "import PySide2._cx_freeze_resource\n"
    code_string += "import PySide2._cx_freeze_qt_debug\n"
    code_string += "# cx_Freeze patch end\n"
    module.code = compile(code_string, os.fspath(module.file), "exec")


__all__ = [
    "load_pyside2",
    "load_pyside2_qt",
    "load_pyside2_qtcharts",
    "load_pyside2_qtdatavisualization",
    "load_pyside2_qtdesigner",
    "load_pyside2_qtgui",
    "load_pyside2_qthelp",
    "load_pyside2_qtlocation",
    "load_pyside2_qtmultimedia",
    "load_pyside2_qtmultimediawidgets",
    "load_pyside2_qtopengl",
    "load_pyside2_qtpositioning",
    "load_pyside2_qtprintsupport",
    "load_pyside2_qtqml",
    "load_pyside2_qtscripttools",
    "load_pyside2_qtsql",
    "load_pyside2_qtsvg",
    "load_pyside2_qttest",
    "load_pyside2_qtuitools",
    "load_pyside2_qtwebengine",
    "load_pyside2_qtwebenginecore",
    "load_pyside2_qtwebenginewidgets",
    "load_pyside2_qtwebkit",
    "load_pyside2_qtwebsockets",
    "load_pyside2_qtwidgets",
    "load_pyside2_qtxmlpatterns",
    "load_pyside2_uic",
]



"""Initialization script for cx_Freeze which imports the site module (as per
normal processing of a Python script) and then searches for a file with the
same name as the shared library but with the extension .pth. The entries in
this file are used to modify the path to use for subsequent imports.
"""
from __future__ import annotations

import os
import sys

# the site module must be imported for normal behavior to take place; it is
# done dynamically so that cx_Freeze will not add all modules referenced by
# the site module to the frozen executable
__import__("site")

# now locate the pth file to modify the path appropriately
name, ext = os.path.splitext(sys.executable)
filename = name + ".pth"
with open(filename, encoding="utf-8") as in_file:
    sys.path = [s.strip() for s in in_file.read().splitlines()] + sys.path


def run():  # noqa: D103
    pass



"""Module for the Executable base class."""

from __future__ import annotations

import os
import string
import sys
from pathlib import Path
from sysconfig import get_config_var, get_platform

from setuptools import Distribution

from ._compat import IS_MINGW, IS_WINDOWS
from .common import get_resource_file_path
from .exception import OptionError, SetupError

STRINGREPLACE = list(
    string.whitespace + string.punctuation.replace(".", "").replace("_", "")
)

__all__ = ["Executable"]


class Executable:
    """Base Executable class."""

    def __init__(
        self,
        script: str | Path,
        init_script: str | Path | None = None,
        base: str | Path | None = None,
        target_name: str | None = None,
        icon: str | Path | None = None,
        shortcut_name: str | None = None,
        shortcut_dir: str | Path | None = None,
        copyright: str | None = None,  # noqa: A002
        trademarks: str | None = None,
        manifest: str | Path | None = None,
        uac_admin: bool = False,
    ):
        self.main_script = script
        self.init_script = init_script
        self.base = base
        self.target_name = target_name
        self.icon = icon
        self.shortcut_name = shortcut_name
        self.shortcut_dir = shortcut_dir
        self.copyright = copyright
        self.trademarks = trademarks
        self.manifest = manifest
        self.uac_admin = uac_admin

    def __repr__(self):
        return f"<Executable script={self.main_script}>"

    @property
    def base(self) -> Path:
        """:return: the name of the base executable
        :rtype: Path

        """
        return self._base

    @base.setter
    def base(self, name: str | Path | None):
        name = name or "console"
        if IS_WINDOWS or IS_MINGW:
            platform_nodot = get_platform().replace(".", "").replace("-", "_")
            soabi = f"{sys.implementation.cache_tag}-{platform_nodot}"
            suffix = ".exe"
        else:
            soabi = get_config_var("SOABI")
            suffix = ""
        name_base = f"{name}-{soabi}"
        self._base: Path = get_resource_file_path("bases", name_base, suffix)
        if self._base is None:
            raise OptionError(f"no base named {name!r} ({name_base!r})")
        self._ext: str = suffix

    @property
    def icon(self) -> str:
        """:return: the path of the icon
        :rtype: Path

        """
        return self._icon

    @icon.setter
    def icon(self, name: str | Path | None):
        self._icon: Path = Path(name) if name else None

    @property
    def init_module_name(self) -> str:
        """:return: the name of the init module in zip file
        :rtype: str

        """
        return f"{self._internal_name}__init__"

    @property
    def init_script(self) -> Path:
        """:return: the name of the initialization script that will be executed
        before the main script is executed
        :rtype: Path

        """
        return self._init_script

    @init_script.setter
    def init_script(self, name: str | Path | None):
        name = name or "console"
        self._init_script: Path = get_resource_file_path(
            "initscripts", name, ".py"
        )
        if self._init_script is None:
            raise OptionError(f"no init_script named {name}")

    @property
    def main_module_name(self) -> str:
        """:return: the name of the main module in zip file
        :rtype: str

        """
        return f"{self._internal_name}__main__"

    @property
    def main_script(self) -> Path:
        """:return: the path of the file containing the script which is to be
        frozen
        :rtype: Path

        """
        return self._main_script

    @main_script.setter
    def main_script(self, name: str | Path):
        self._main_script: Path = Path(name)

    @property
    def manifest(self) -> str | None:
        """:return: the XML schema of the manifest which is to be included in
        the frozen executable
        :rtype: str

        """
        return self._manifest

    @manifest.setter
    def manifest(self, name: str | Path | None) -> None:
        self._manifest: str | None = None
        if name is None:
            return
        if isinstance(name, str):
            name = Path(name)
        self._manifest = name.read_text(encoding="utf-8")

    @property
    def shortcut_name(self) -> str:
        """:return: the name to give a shortcut for the executable when
        included in an MSI package (Windows only).
        :rtype: str

        """
        return self._shortcut_name

    @shortcut_name.setter
    def shortcut_name(self, name: str):
        self._shortcut_name: str = name

    @property
    def shortcut_dir(self) -> Path:
        """:return: tthe directory in which to place the shortcut when being
        installed by an MSI package; see the MSI Shortcut table documentation
        for more information on what values can be placed here (Windows only).
        :rtype: Path

        """
        return self._shortcut_dir

    @shortcut_dir.setter
    def shortcut_dir(self, name: str | Path):
        self._shortcut_dir: Path = Path(name) if name else None

    @property
    def target_name(self) -> str:
        """:return: the name of the target executable
        :rtype: str

        """
        return self._name + self._ext

    @target_name.setter
    def target_name(self, name: str | None):
        if name is None:
            name = self.main_script.stem
        else:
            pathname = Path(name)
            if name != pathname.name:
                raise OptionError(
                    "target_name should only be the name, for example: "
                    f"{pathname.name}"
                )
            if sys.platform == "win32" and pathname.suffix.lower() == ".exe":
                name = pathname.stem
        self._name: str = name
        name = name.partition(".")[0]
        if not name.isidentifier():
            for invalid in STRINGREPLACE:
                name = name.replace(invalid, "_")
        name = os.path.normcase(name)
        if not name.isidentifier():
            raise OptionError(f"Invalid name for target_name ({self._name!r})")
        self._internal_name: str = name


def validate_executables(dist: Distribution, attr: str, value):  # noqa: ARG001
    """Verify that value is a Executable list."""
    try:
        # verify that value is a list or tuple to exclude unordered
        # or single-use iterables
        assert isinstance(value, (list, tuple))
        # verify that elements of value are Executable
        for executable in value:
            assert isinstance(executable, Executable)
    except (TypeError, ValueError, AttributeError, AssertionError) as exc:
        raise SetupError(
            f"{attr!r} must be a list of Executable (got {value!r})"
        ) from exc



"""Implements `Parser` interface to create an abstraction to parse binary
files.
"""

from __future__ import annotations

import os
import re
import shutil
import stat
import sys
from abc import ABC, abstractmethod
from contextlib import suppress
from pathlib import Path
from subprocess import CalledProcessError, check_call, check_output, run
from typing import Any

from ._compat import IS_MINGW, IS_WINDOWS
from .common import TemporaryPath

# In Windows, to get dependencies, the default is to use lief package,
# but LIEF can be disabled with:
# set CX_FREEZE_BIND=imagehlp
if IS_WINDOWS or IS_MINGW:
    import lief

    with suppress(ImportError):
        from .util import BindError, GetDependentFiles

    lief.logging.set_level(lief.logging.LOGGING_LEVEL.ERROR)

LIEF_DISABLED = os.environ.get("CX_FREEZE_BIND", "") == "imagehlp"
PE_EXT = (".exe", ".dll", ".pyd")
MAGIC_ELF = b"\x7fELF"
NON_ELF_EXT = ".a:.c:.h:.py:.pyc:.pyi:.pyx:.pxd:.txt:.html:.xml".split(":")
NON_ELF_EXT += ".png:.jpg:.gif:.jar:.json".split(":")


class Parser(ABC):
    """`Parser` interface."""

    def __init__(self, silent: int = 0) -> None:
        self.dependent_files: dict[Path, set[Path]] = {}
        self._silent: int = silent

    @abstractmethod
    def get_dependent_files(self, path: str | Path) -> set[Path]:
        """Return the file's dependencies using platform-specific tools
        (lief package or the imagehlp library on Windows, otool on Mac OS X or
        ldd on Linux); limit this list by the exclusion lists as needed.
        (Implemented separately for each platform.)
        .
        """


class PEParser(Parser):
    """`PEParser` is based on the `lief` package. If it is disabled,
    use the old friend `cx_Freeze.util` extension module.
    """

    @staticmethod
    def is_pe(path: str | Path) -> bool:
        """Determines whether the file is a PE file."""
        if isinstance(path, str):
            path = Path(path)
        return path.suffix.lower().endswith(PE_EXT) and path.is_file()

    def _get_dependent_files_lief(self, path: Path) -> set[Path]:
        with path.open("rb", buffering=0) as raw:
            binary = lief.PE.parse(raw, path.name)
        if not binary:
            return set()

        libraries: list[str] = []
        if binary.has_imports:
            libraries += binary.libraries
        for delay_import in binary.delay_imports:
            libraries.append(delay_import.name)

        dependent_files: set[Path] = set()
        orig_path: list[str] = os.environ["PATH"]
        search_path: list[str] = sys.path + orig_path.split(os.pathsep)
        for library_name in libraries:
            for directory in search_path:
                library_path = Path(directory, library_name)
                if library_path.is_file():
                    dependent_files.add(library_path)
                    break
        return dependent_files

    def _get_dependent_files_imagehlp(self, path: Path) -> set[Path]:
        dependent_files: set[Path] = set()
        orig_path = os.environ["PATH"]
        os.environ["PATH"] = os.pathsep.join(sys.path) + os.pathsep + orig_path
        try:
            files: list[str] = GetDependentFiles(path)
        except BindError as exc:
            # Sometimes this gets called when path is not actually
            # a library (See issue 88).
            if self._silent < 3:
                print("WARNING: ignoring error during ", end="")
                print(f"GetDependentFiles({path}):", exc)
        else:
            dependent_files = {Path(dep) for dep in files}
        os.environ["PATH"] = orig_path
        return dependent_files

    if LIEF_DISABLED:
        _get_dependent_files = _get_dependent_files_imagehlp
    else:
        _get_dependent_files = _get_dependent_files_lief

    def get_dependent_files(self, path: str | Path) -> set[Path]:
        if isinstance(path, str):
            path = Path(path)
        with suppress(KeyError):
            return self.dependent_files[path]
        if not self.is_pe(path):
            return set()

        dependent_files: set[Path] = self._get_dependent_files(path)
        self.dependent_files[path] = dependent_files
        return dependent_files

    def read_manifest(self, path: str | Path) -> str:
        """:return: the XML schema of the manifest included in the executable
        :rtype: str

        """
        if isinstance(path, str):
            path = Path(path)
        with path.open("rb", buffering=0) as raw:
            binary = lief.PE.parse(raw, path.name)
        try:
            resources_manager = binary.resources_manager
            manifest = resources_manager.manifest
        except lief.exception as exc:
            raise RuntimeError(exc) from None
        return manifest

    def write_manifest(self, path: str | Path, manifest: str) -> None:
        """:return: write the XML schema of the manifest into the executable
        :rtype: str

        """
        if isinstance(path, str):
            path = Path(path)
        with path.open("rb", buffering=0) as raw:
            binary = lief.PE.parse(raw, path.name)
        try:
            resources_manager = binary.resources_manager
            resources_manager.manifest = manifest
            builder = lief.PE.Builder(binary)
            builder.build_resources(True)
            builder.build()
            with TemporaryPath("temp.exe") as tmp_path:
                builder.write(os.fspath(tmp_path))
                tmp_path.replace(path)
        except lief.exception as exc:
            raise RuntimeError(exc) from None


class ELFParser(Parser):
    """`ELFParser` is based on the logic around invoking `patchelf` and
    `ldd`.
    """

    def __init__(self, silent: int = 0) -> None:
        super().__init__(silent)
        self.linker_warnings: dict[Path, Any] = {}
        _verify_patchelf()

    @staticmethod
    def is_elf(path: str | Path) -> bool:
        """Check if the executable is an ELF."""
        if isinstance(path, str):
            path = Path(path)
        if (
            path.suffix in NON_ELF_EXT
            or path.is_symlink()
            or not path.is_file()
        ):
            return False
        with open(path, "rb") as binary:
            four_bytes = binary.read(4)
        return bool(four_bytes == MAGIC_ELF)

    def get_dependent_files(self, path: str | Path) -> set[Path]:
        if isinstance(path, str):
            path = Path(path)
        with suppress(KeyError):
            return self.dependent_files[path]

        dependent_files: set[Path] = set()
        if not self.is_elf(path) or not os.access(path, os.X_OK):
            return dependent_files

        split_string = " => "
        dependent_file_index = 1
        args = ("ldd", path)
        process = run(args, check=False, capture_output=True, encoding="utf-8")
        for line in process.stdout.splitlines():
            parts = line.expandtabs().strip().split(split_string)
            if len(parts) != 2:
                continue
            dependent_file = parts[dependent_file_index].strip()
            if dependent_file == path.name:
                continue
            if dependent_file in ("not found", "(file not found)"):
                filename = parts[0]
                if filename not in self.linker_warnings:
                    self.linker_warnings[filename] = None
                    if self._silent < 3:
                        print(f"WARNING: cannot find '{filename}'")
                continue
            if dependent_file.startswith("("):
                continue
            pos = dependent_file.find(" (")
            if pos >= 0:
                dependent_file = dependent_file[:pos].strip()
            if dependent_file:
                dependent_files.add(Path(dependent_file))
        if process.returncode and self._silent < 3:
            print("WARNING:", *args, "returns:")
            print(process.stderr, end="")
        self.dependent_files[path] = dependent_files
        return dependent_files

    def get_rpath(self, filename: str | Path) -> str:
        """Gets the rpath of the executable."""
        args = ["patchelf", "--print-rpath", filename]
        with suppress(CalledProcessError):
            return check_output(args, encoding="utf-8").strip()
        return ""

    def replace_needed(
        self, filename: str | Path, so_name: str, new_so_name: str
    ) -> None:
        """Replace DT_NEEDED entry in the dynamic table."""
        self._set_write_mode(filename)
        args = ["patchelf", "--replace-needed", so_name, new_so_name, filename]
        check_call(args)

    def set_rpath(self, filename: str | Path, rpath: str) -> None:
        """Sets the rpath of the executable."""
        self._set_write_mode(filename)
        args = ["patchelf", "--remove-rpath", filename]
        check_call(args)
        args = ["patchelf", "--force-rpath", "--set-rpath", rpath, filename]
        check_call(args)

    def set_soname(self, filename: str | Path, new_so_name: str) -> None:
        """Sets DT_SONAME entry in the dynamic table."""
        self._set_write_mode(filename)
        args = ["patchelf", "--set-soname", new_so_name, filename]
        check_call(args)

    @staticmethod
    def _set_write_mode(filename: str | Path) -> None:
        if isinstance(filename, str):
            filename = Path(filename)
        mode = filename.stat().st_mode
        if mode & stat.S_IWUSR == 0:
            filename.chmod(mode | stat.S_IWUSR)


def _verify_patchelf() -> None:
    """Looks for the ``patchelf`` external binary in the PATH, checks for the
    required version, and throws an exception if a proper version can't be
    found. Otherwise, silence is golden.
    """
    if not shutil.which("patchelf"):
        raise ValueError("Cannot find required utility `patchelf` in PATH")
    try:
        version = check_output(["patchelf", "--version"], encoding="utf-8")
    except CalledProcessError:
        raise ValueError("Could not call `patchelf` binary") from None

    mobj = re.match(r"patchelf\s+(\d+(.\d+)?)", version)
    if mobj and tuple(int(x) for x in mobj.group(1).split(".")) >= (0, 14):
        return
    raise ValueError(
        f"patchelf {version} found. cx_Freeze requires patchelf >= 0.14."
    )



from __future__ import annotations

import sys
import time
from contextlib import AbstractContextManager

import zmq


class ignore(AbstractContextManager):
    def __init__(self, call):
        self.call = call

    def __enter__(self):
        if hasattr(self.call, "__enter__"):
            return self.call
        return self

    def __exit__(self, *exc_info):
        if hasattr(self.call, "__exit__"):
            return self.call.__exit__
        return None


if len(sys.argv) > 1:
    port = int(sys.argv[1])
else:
    port = 5556

url = f"tcp://*:{port}"
reply = "Pong"

context = zmq.Context()
socket = context.socket(zmq.REP)

print(f"Server listening at {url}")
try:
    with ignore(socket.bind(url)):
        while True:
            message = socket.recv_string()
            print(f"Server received: {message}")
            if message == "Close":
                reply = "Closing"
            else:
                time.sleep(1)
            print(f"Server sending: {reply}")
            socket.send_string(reply)
            if message == "Close":
                break
except KeyboardInterrupt:
    sys.exit(0)
